﻿--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8002)
				select 0, 'Already correct database version'
            else if (@ver = 8001)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery

GO
PRINT N'Altering Table[dbo].[tblContentType]...';

GO
ALTER TABLE [dbo].[tblContentType]
	ADD [SupportedMediaExtensions] NVARCHAR(255) NULL


GO
PRINT N'Altering Procedure [dbo].[netContentTypeSave]...';


GO
ALTER PROCEDURE [dbo].[netContentTypeSave]
(
	@ContentTypeID			    INT,
	@ContentTypeGUID		    UNIQUEIDENTIFIER,
	@Saved					    DATETIME		= NULL,
	@SavedBy				    NVARCHAR(255)	= NULL,
	@Name					    NVARCHAR(50),
	@Base					    NVARCHAR(50)	= NULL,
	@Version				    NVARCHAR(50)	= NULL,
	@DisplayName			    NVARCHAR(50)	= NULL,
	@Description			    NVARCHAR(255)	= NULL,
	@DefaultWebFormTemplate	    NVARCHAR(1024)	= NULL,
	@DefaultMvcController	    NVARCHAR(1024)	= NULL,
	@DefaultMvcPartialView	    NVARCHAR(255)	= NULL,
	@Filename				    NVARCHAR(255)	= NULL,
	@Available				    BIT				= NULL,
	@SortOrder				    INT				= NULL,
	@ModelType				    NVARCHAR(1024)	= NULL,
	
	@DefaultID				    INT				= NULL,
	@DefaultName 			    NVARCHAR(100)	= NULL,
	@StartPublishOffset		    INT				= NULL,
	@StopPublishOffset		    INT				= NULL,
	@VisibleInMenu			    BIT				= NULL,
	@PeerOrder 				    INT				= NULL,
	@ChildOrderRule 		    INT				= NULL,
	@ArchiveContentID 		    INT				= NULL,
	@FrameID 				    INT				= NULL,
	@ACL					    NVARCHAR(MAX)	= NULL,
	@ContentType			    INT				= 0,
	@Created				    DATETIME,
    @SupportedMediaExtensions   NVARCHAR(255)   = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	DECLARE @IdString NVARCHAR(255)
	
	IF @ContentTypeID <= 0
	BEGIN
        --We want to block parallel inserts which can happen if several applications initializes in parallel, hence we read all table with update lock
        DECLARE @NotUsed INT
        SELECT @NotUsed = pkID FROM tblContentType WITH (UPDLOCK, HOLDLOCK)
        
		SET @ContentTypeID = ISNULL((SELECT pkID FROM tblContentType where Name = @Name), @ContentTypeID)
	END

	IF (@ContentTypeID <= 0)
	BEGIN
		SELECT TOP 1 @IdString = IdString FROM tblContentType
		INSERT INTO tblContentType
			(Saved,
			SavedBy,
			Name,
			DisplayName,
			Base,
			Version,
			DefaultMvcController,
			DefaultWebFormTemplate,
			DefaultMvcPartialView,
			Description,
			Available,
			SortOrder,
			ModelType,
			Filename,
			IdString,
			ContentTypeGUID,
			ACL,
			ContentType,
			Created,
			SupportedMediaExtensions)
		VALUES
			(@Saved,
			@SavedBy,
			@Name,
			@DisplayName,
			@Base,
			@Version,
			@DefaultMvcController,
			@DefaultWebFormTemplate,
			@DefaultMvcPartialView,
			@Description,
			@Available,
			@SortOrder,
			@ModelType,
			@Filename,
			@IdString,
			@ContentTypeGUID,
			@ACL,
			@ContentType,
			@Created,
			@SupportedMediaExtensions)

		SET @ContentTypeID = SCOPE_IDENTITY() 
		
	END
	ELSE
	BEGIN
		UPDATE tblContentType
		SET
			Saved = @Saved,
			SavedBy = @SavedBy,
			Name=@Name,
			Base=@Base,
			Version=@Version,
			DisplayName=@DisplayName,
			Description=@Description,
			DefaultWebFormTemplate=@DefaultWebFormTemplate,
			DefaultMvcController=@DefaultMvcController,
			DefaultMvcPartialView=@DefaultMvcPartialView,
			Available=@Available,
			SortOrder=@SortOrder,
			ModelType = @ModelType,
			Filename = @Filename,
			ACL=@ACL,
			ContentType = @ContentType,
			@ContentTypeGUID = ContentTypeGUID,
			SupportedMediaExtensions = @SupportedMediaExtensions
		WHERE
			pkID = @ContentTypeID
	END

    IF (@DefaultID IS NULL)
	BEGIN
		DELETE FROM tblContentTypeDefault WHERE fkContentTypeID=@ContentTypeID
	END
	ELSE
	BEGIN
		IF (EXISTS (SELECT pkID FROM tblContentTypeDefault WHERE fkContentTypeID=@ContentTypeID))
		BEGIN
			UPDATE tblContentTypeDefault SET
				Name 				= @DefaultName,
				StartPublishOffset 	= @StartPublishOffset,
				StopPublishOffset 	= @StopPublishOffset,
				VisibleInMenu 		= @VisibleInMenu,
				PeerOrder 			= @PeerOrder,
				ChildOrderRule 		= @ChildOrderRule,
				fkArchiveContentID 	= @ArchiveContentID,
				fkFrameID 			= @FrameID
			WHERE fkContentTypeID = @ContentTypeID
		END
		ELSE
		BEGIN
			INSERT INTO tblContentTypeDefault 
				(fkContentTypeID,
				Name,
				StartPublishOffset,
				StopPublishOffset,
				VisibleInMenu,
				PeerOrder,
				ChildOrderRule,
				fkArchiveContentID,
				fkFrameID)
			VALUES
				(@ContentTypeID,
				@DefaultName,
				@StartPublishOffset,
				@StopPublishOffset,
				@VisibleInMenu,
				@PeerOrder,
				@ChildOrderRule,
				@ArchiveContentID,
				@FrameID)
		END
	END
    
	SELECT @ContentTypeID AS "ID", @ContentTypeGUID AS "GUID"
END
GO
PRINT N'Altering Procedure [dbo].[netContentTypeList]...';

GO
ALTER PROCEDURE [dbo].[netContentTypeList]
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT	CT.pkID AS ID,
			CONVERT(NVARCHAR(38),CT.ContentTypeGUID) AS Guid,
			CT.Created,
			CT.Saved,
			CT.SavedBy,
			CT.Name,
			CT.Base,
			CT.Version,
			CT.DisplayName,
			CT.Description,
			CT.DefaultWebFormTemplate,
			CT.DefaultMvcController,
			CT.DefaultMvcPartialView,
			CT.Available,
			CT.SortOrder,
			CT.ModelType,
			CT.Filename,
			CT.ACL,
			CT.ContentType,
            CT.SupportedMediaExtensions,
			CTD.pkID AS DefaultID,
			CTD.Name AS DefaultName,
			CTD.StartPublishOffset,
			CTD.StopPublishOffset,
			CONVERT(INT,CTD.VisibleInMenu) AS VisibleInMenu,
			CTD.PeerOrder,
			CTD.ChildOrderRule,
			CTD.fkFrameID AS FrameID,
			CTD.fkArchiveContentID AS ArchiveContentLink
	FROM tblContentType CT
	LEFT JOIN tblContentTypeDefault AS CTD ON CTD.fkContentTypeID=CT.pkID 
	ORDER BY CT.SortOrder
END

GO
PRINT N'Altering Table[dbo].[tblPropertyDefinition]...';

GO
ALTER TABLE [dbo].[tblPropertyDefinition]
    ADD [ImageDescriptor] NVARCHAR(50) NULL

GO
PRINT N'Altering View [dbo].[netPageDefinition]...';

GO
ALTER VIEW [dbo].[tblPageDefinition]
AS
SELECT  [pkID],
		[fkContentTypeID] AS fkPageTypeID,
		[fkPropertyDefinitionTypeID] AS fkPageDefinitionTypeID,
		[FieldOrder],
		[Name],
		[Property],
		[Required],
		[Advanced],
		[Searchable],
		[EditCaption],
		[HelpText],
		[ObjectProgID],
		[DefaultValueType],
		[LongStringSettings],
		[SettingsID],
		[LanguageSpecific],
		[DisplayEditUI],
		[ExistsOnModel],
        [EditorHint],
        [ImageDescriptor]
FROM    dbo.tblPropertyDefinition


GO
PRINT N'Altering Procedure [dbo].[netPageDefinitionGet]...';

GO
ALTER PROCEDURE [dbo].[netPageDefinitionGet]
(
	@PageDefinitionID INT
)
AS
BEGIN
	SELECT tblPageDefinition.pkID AS ID,
		fkPageTypeID AS PageTypeID,
		COALESCE(fkPageDefinitionTypeID,tblPageDefinition.Property) AS PageDefinitionTypeID,
		tblPageDefinition.Name,
		COALESCE(tblPageDefinitionType.Property,tblPageDefinition.Property) AS Type,
		CONVERT(INT,Required) AS Required,
		Advanced,
		CONVERT(INT,Searchable) AS Searchable,
		DefaultValueType,
		EditCaption,
		HelpText,
		ObjectProgID,
		LongStringSettings,
		SettingsID,
		CONVERT(INT,Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		PageType,
		PageLink,
		Date AS DateValue,
		String,
		LongString,
		FieldOrder,
		LanguageSpecific,
		DisplayEditUI,
		ExistsOnModel,
        EditorHint,
        ImageDescriptor
	FROM tblPageDefinition
	LEFT JOIN tblPropertyDefault ON tblPropertyDefault.fkPageDefinitionID=tblPageDefinition.pkID
	LEFT JOIN tblPageDefinitionType ON tblPageDefinitionType.pkID=tblPageDefinition.fkPageDefinitionTypeID
	WHERE tblPageDefinition.pkID = @PageDefinitionID
END

GO
PRINT N'Altering Procedure [dbo].[netPageDefinitionList]...';

GO

ALTER PROCEDURE [dbo].[netPageDefinitionList]
(
	@PageTypeID INT = NULL
)
AS
BEGIN
	SELECT tblPageDefinition.pkID AS ID,
		fkPageTypeID AS PageTypeID,
		COALESCE(fkPageDefinitionTypeID,tblPageDefinition.Property) AS PageDefinitionTypeID,
		tblPageDefinition.Name,
		COALESCE(tblPageDefinitionType.Property,tblPageDefinition.Property) AS Type,
		CONVERT(INT,Required) AS Required,
		Advanced,
		CONVERT(INT,Searchable) AS Searchable,
		DefaultValueType,
		EditCaption,
		HelpText,
		ObjectProgID,
		LongStringSettings,
		SettingsID,
		CONVERT(INT,Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		PageType,
		PageLink,
		Date AS DateValue,
		String,
		LongString,
		NULL AS OldType,
		FieldOrder,
		LanguageSpecific,
		DisplayEditUI,
		ExistsOnModel,
        EditorHint,
        ImageDescriptor
	FROM tblPageDefinition
	LEFT JOIN tblPropertyDefault ON tblPropertyDefault.fkPageDefinitionID=tblPageDefinition.pkID
	LEFT JOIN tblPageDefinitionType ON tblPageDefinitionType.pkID=tblPageDefinition.fkPageDefinitionTypeID
	WHERE (fkPageTypeID = @PageTypeID) OR (fkPageTypeID IS NULL AND @PageTypeID IS NULL)
	ORDER BY FieldOrder,tblPageDefinition.pkID
END

GO
PRINT N'Altering Procedure [dbo].[netPageDefinitionSave]...';

GO

ALTER PROCEDURE dbo.netPageDefinitionSave
(
	@PageDefinitionID      INT OUTPUT,
	@PageTypeID            INT,
	@Name                  NVARCHAR(100),
	@PageDefinitionTypeID  INT,
	@Required              BIT = NULL,
	@Advanced              INT = NULL,
	@Searchable            BIT = NULL,
	@DefaultValueType      INT = NULL,
	@EditCaption           NVARCHAR(255) = NULL,
	@HelpText              NVARCHAR(2000) = NULL,
	@ObjectProgID          NVARCHAR(255) = NULL,
	@LongStringSettings    INT = NULL,
	@SettingsID            UNIQUEIDENTIFIER = NULL,
	@FieldOrder            INT = NULL,
	@Type                  INT = NULL OUTPUT,
	@OldType               INT = NULL OUTPUT,
	@LanguageSpecific      INT = 0,
	@DisplayEditUI         BIT = NULL,
	@ExistsOnModel         BIT = 0,
    @EditorHint            NVARCHAR(255) = NULL,
    @ImageDescriptor       NVARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	SELECT @OldType = tblPropertyDefinitionType.Property 
	FROM tblPropertyDefinition
	INNER JOIN tblPropertyDefinitionType ON tblPropertyDefinitionType.pkID=tblPropertyDefinition.fkPropertyDefinitionTypeID
	WHERE tblPropertyDefinition.pkID=@PageDefinitionID

	SELECT @Type = Property FROM tblPropertyDefinitionType WHERE pkID=@PageDefinitionTypeID
	IF @Type IS NULL
		RAISERROR('Cannot find data type',16,1)
	IF @PageTypeID=0
		SET @PageTypeID = NULL

	IF @PageDefinitionID = 0 AND @ExistsOnModel = 1
	BEGIN
        --We want to block parallel inserts which can happen if several applications initializes in parallel, hence we read all table with update lock
        DECLARE @NotUsed INT
        SELECT @NotUsed = pkID FROM tblPropertyDefinition WITH (UPDLOCK, HOLDLOCK)
		
        SET @PageDefinitionID = ISNULL((SELECT pkID FROM tblPropertyDefinition where Name = @Name AND fkContentTypeID = @PageTypeID), @PageDefinitionID)
	END

	IF @PageDefinitionID=0
	BEGIN	
		INSERT INTO tblPropertyDefinition
		(
			fkContentTypeID,
			fkPropertyDefinitionTypeID,
			Name,
			Property,
			Required,
			Advanced,
			Searchable,
			DefaultValueType,
			EditCaption,
			HelpText,
			ObjectProgID,
			LongStringSettings,
			SettingsID,
			FieldOrder,
			LanguageSpecific,
			DisplayEditUI,
			ExistsOnModel,
            EditorHint,
            ImageDescriptor
		)
		VALUES
		(
			@PageTypeID,
			@PageDefinitionTypeID,
			@Name,
			@Type,
			@Required,
			@Advanced,
			@Searchable,
			@DefaultValueType,
			@EditCaption,
			@HelpText,
			@ObjectProgID,
			@LongStringSettings,
			@SettingsID,
			@FieldOrder,
			@LanguageSpecific,
			@DisplayEditUI,
			@ExistsOnModel,
            @EditorHint,
            @ImageDescriptor
		)
		SET @PageDefinitionID =  SCOPE_IDENTITY() 
	END
	ELSE
	BEGIN
		UPDATE tblPropertyDefinition SET
			Name 		= @Name,
			fkPropertyDefinitionTypeID	= @PageDefinitionTypeID,
			Property 	= @Type,
			Required 	= @Required,
			Advanced 	= @Advanced,
			Searchable 	= @Searchable,
			DefaultValueType = @DefaultValueType,
			EditCaption 	= @EditCaption,
			HelpText 	= @HelpText,
			ObjectProgID 	= @ObjectProgID,
			LongStringSettings = @LongStringSettings,
			SettingsID = @SettingsID,
			LanguageSpecific = @LanguageSpecific,
			FieldOrder = @FieldOrder,
			DisplayEditUI = @DisplayEditUI,
			ExistsOnModel = @ExistsOnModel,
            EditorHint = @EditorHint,
            ImageDescriptor = @ImageDescriptor
		WHERE pkID=@PageDefinitionID

        DELETE FROM tblPropertyDefault WHERE fkPageDefinitionID=@PageDefinitionID
	    IF @LanguageSpecific<3
	    BEGIN
		    /* NOTE: Here we take into consideration that language neutral dynamic properties are always stored on language 
			    with id 1 (which perhaps should be changed and in that case the special handling here could be removed). */
		    IF @PageTypeID IS NULL
		    BEGIN
			    DELETE tblProperty
			    FROM tblProperty
			    INNER JOIN tblPage ON tblPage.pkID=tblProperty.fkPageID
			    WHERE fkPageDefinitionID=@PageDefinitionID AND tblProperty.fkLanguageBranchID<>1
		    END
		    ELSE
		    BEGIN
			    DELETE tblProperty
			    FROM tblProperty
			    INNER JOIN tblPage ON tblPage.pkID=tblProperty.fkPageID
			    WHERE fkPageDefinitionID=@PageDefinitionID AND tblProperty.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID
		    END
		    DELETE tblWorkProperty
		    FROM tblWorkProperty
		    INNER JOIN tblWorkPage ON tblWorkProperty.fkWorkPageID=tblWorkPage.pkID
		    INNER JOIN tblPage ON tblPage.pkID=tblWorkPage.fkPageID
		    WHERE fkPageDefinitionID=@PageDefinitionID AND tblWorkPage.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID

		    DELETE 
			    tblCategoryPage
		    FROM
			    tblCategoryPage
		    INNER JOIN
			    tblPage
		    ON
			    tblPage.pkID = tblCategoryPage.fkPageID
		    WHERE
			    CategoryType = @PageDefinitionID
		    AND
			    tblCategoryPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID

		    DELETE 
			    tblWorkCategory
		    FROM
			    tblWorkCategory
		    INNER JOIN 
			    tblWorkPage
		    ON
			    tblWorkCategory.fkWorkPageID = tblWorkPage.pkID
		    INNER JOIN
			    tblPage
		    ON
			    tblPage.pkID = tblWorkPage.fkPageID
		    WHERE
			    CategoryType = @PageDefinitionID
		    AND
			    tblWorkPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID
	    END
	END
END

GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';


GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8002
GO
PRINT N'Update complete.';


GO
