--beginvalidatingquery
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
begin
	declare @ver int
	exec @ver=sp_DatabaseVersion
	if (@ver >= 8011)
		select 0, 'Already correct database version'
	else if (@ver = 8010)
		select 1, 'Upgrading database'
	else
		select -1, 'Invalid database version detected'
end
else
	select -1, 'Not an EPiServer database'
--endvalidatingquery


GO
PRINT N'Altering Table [dbo].[tblPropertyDefinitionType]...';


GO
ALTER TABLE [dbo].[tblPropertyDefinitionType]
	ADD [DisplayName] NVARCHAR (255) NULL;


GO
PRINT N'Altering Procedure [dbo].[netPropertyDefinitionTypeList]...';


GO

ALTER PROCEDURE dbo.netPropertyDefinitionTypeList
AS
BEGIN
	SELECT	DT.pkID AS ID,
			DT.Name,
			DT.DisplayName,
			DT.Property,
			DT.GUID,
			DT.TypeName,
			DT.AssemblyName, 
			DT.fkContentTypeGUID AS BlockTypeID,
			PT.Name as BlockTypeName,
			PT.ModelType as BlockTypeModel
	FROM tblPropertyDefinitionType as DT
		LEFT OUTER JOIN tblContentType as PT ON DT.fkContentTypeGUID = PT.ContentTypeGUID
	ORDER BY DT.Name
END
GO
PRINT N'Altering Procedure [dbo].[netPropertyDefinitionTypeSave]...';


GO

ALTER PROCEDURE [dbo].[netPropertyDefinitionTypeSave]
(
	@ID				INT OUTPUT,
	@Property		INT,
	@Name			NVARCHAR(255),
	@DisplayName	NVARCHAR(255),
	@GUID			uniqueidentifier = NULL,
	@TypeName		NVARCHAR(255) = NULL,
	@AssemblyName	NVARCHAR(255) = NULL,
	@BlockTypeID	uniqueidentifier = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	/* In case several sites start up at sametime, e.g. in enterprise it may occour that both sites tries to insert at same time. 
	Therefore a check is made to see it it already exist an entry with same guid, and if so an update is performed instead of insert.*/
	IF @ID <= 0
	BEGIN
		--We want to block parallel inserts which can happen if several applications initializes in parallel, hence we read all table with update lock
		DECLARE @NotUsed INT
		SELECT @NotUsed = pkID FROM tblPropertyDefinitionType WITH (UPDLOCK, HOLDLOCK)

		IF (@BlockTypeID IS NOT NULL)
			SET @ID = ISNULL((SELECT pkID FROM tblPropertyDefinitionType WHERE fkContentTypeGUID = @BlockTypeID), @ID)
		ELSE
			SET @ID = ISNULL((SELECT pkID FROM tblPropertyDefinitionType WHERE TypeName = @TypeName AND AssemblyName = @AssemblyName), @ID)
	END

	IF @ID<0
	BEGIN
		IF @AssemblyName='EPiServer'
			SELECT @ID = Max(pkID)+1 FROM tblPropertyDefinitionType WHERE pkID<1000
		ELSE
			SELECT @ID = CASE WHEN Max(pkID)<1000 THEN 1000 ELSE Max(pkID)+1 END FROM tblPropertyDefinitionType
		INSERT INTO tblPropertyDefinitionType
		(
			pkID,
			Property,
			Name,
			DisplayName,
			GUID,
			TypeName,
			AssemblyName,
			fkContentTypeGUID
		)
		VALUES
		(
			@ID,
			@Property,
			@Name,
			@DisplayName,
			@GUID,
			@TypeName,
			@AssemblyName,
			@BlockTypeID
		)
	END
	ELSE
		UPDATE tblPropertyDefinitionType SET
			Name 				= @Name,
			DisplayName			= @DisplayName,
			Property			= @Property,
			GUID				= @GUID,
			TypeName			= @TypeName,
			AssemblyName		= @AssemblyName,
			fkContentTypeGUID	= @BlockTypeID
		WHERE pkID=@ID
		
END

GO
PRINT N'Altering Procedure [dbo].[netProjectItemGet]...';

GO
ALTER PROCEDURE [dbo].[netProjectItemGet]
	@ProjectID INT,
	@StartIndex INT = 0,
	@MaxCount INT = 0,
	@Category VARCHAR(2555) = NULL,
	@Language VARCHAR(20) = NULL,
	@Status INT = NULL,
	@SortBy VARCHAR(50) = NULL,
	@SortOrder VARCHAR(4) = NULL
AS

BEGIN
	SET NOCOUNT ON;

	SELECT
		tblProjectItem.pkID,
		tblProjectItem.fkProjectID,
		tblProjectItem.ContentLinkID,
		tblProjectItem.ContentLinkWorkID,
		tblProjectItem.ContentLinkProvider,
		tblProjectItem.Language,
		tblProjectItem.Category

	FROM
		tblProjectItem
		LEFT OUTER JOIN tblWorkContent
		ON tblWorkContent.pkID = ContentLinkWorkID AND tblWorkContent.fkContentID = tblProjectItem.ContentLinkID
		LEFT OUTER JOIN tblContent
		ON tblContent.pkID = tblWorkContent.fkContentID
		LEFT OUTER JOIN tblContentType
		ON tblContentType.pkID = tblContent.fkContentTypeID

	WHERE
		(@Category IS NULL OR tblProjectItem.Category = @Category) AND
		(@Language IS NULL OR tblProjectItem.Language = @Language OR tblProjectItem.Language = '') AND -- Should also load invariant languages
		(@Status IS NULL OR tblWorkContent.Status = @Status) AND
		tblProjectItem.fkProjectID = @ProjectID

	ORDER BY
		CASE WHEN @SortBy = 'Name' AND @SortOrder = 'ASC' THEN tblWorkContent.Name END,
		CASE WHEN @SortBy = 'Name' AND @SortOrder = 'DESC' THEN tblWorkContent.Name END DESC,
		CASE WHEN @SortBy = 'Status' AND @SortOrder = 'ASC' THEN tblContent.Deleted END,
		CASE WHEN @SortBy = 'Status' AND @SortOrder = 'ASC' THEN tblWorkContent.Status END,
		CASE WHEN @SortBy = 'Status' AND @SortOrder = 'DESC' THEN tblContent.Deleted END DESC,
		CASE WHEN @SortBy = 'Status' AND @SortOrder = 'DESC' THEN tblWorkContent.Status END DESC,
		CASE WHEN @SortBy = 'ContentType' AND @SortOrder = 'ASC' THEN tblContentType.Base END,
		CASE WHEN @SortBy = 'ContentType' AND @SortOrder = 'DESC' THEN tblContentType.Base END DESC,
		CASE WHEN @SortBy = 'Modified' AND @SortOrder = 'ASC' THEN tblWorkContent.Saved END,
		CASE WHEN @SortBy = 'Modified' AND @SortOrder = 'DESC' THEN tblWorkContent.Saved END DESC,
		CASE WHEN @SortBy <> 'Name' THEN tblWorkContent.Name END

	OFFSET @StartIndex ROWS FETCH NEXT @MaxCount ROWS ONLY

	SELECT
		COUNT(tblProjectItem.pkID) as TotalCount 
	
	FROM
		tblProjectItem
		LEFT OUTER JOIN tblWorkContent
		ON tblWorkContent.pkID = ContentLinkWorkID AND tblWorkContent.fkContentID = tblProjectItem.ContentLinkID
	
	WHERE
		(@Category IS NULL OR tblProjectItem.Category = @Category) AND
		(@Language IS NULL OR tblProjectItem.Language = @Language OR tblProjectItem.Language = '') AND -- Should also load invariant languages
		(@Status IS NULL OR tblWorkContent.Status = @Status) AND
		tblProjectItem.fkProjectID = @ProjectID
END

GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';

GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8011

GO
PRINT N'Update complete.';

GO
