--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8012)
				select 0, 'Already correct database version'
            else if (@ver = 8011)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery


GO
PRINT N'Dropping Function [dbo].[ConvertIndexedScopeName]...';


GO
DROP FUNCTION [dbo].[ConvertIndexedScopeName];


GO
PRINT N'Dropping Procedure [dbo].[netContentLoadPropertyList]...';


GO
DROP PROCEDURE [dbo].[netContentLoadPropertyList];


GO
PRINT N'Creating Function [dbo].[ConvertIndexedScopeName]...';


GO
CREATE FUNCTION [dbo].[ConvertIndexedScopeName]
(
	@ScopeName nvarchar(450),
	@OldDefinitionID int,
	@NewDefinitionID int,
    @ScopeDelimiter char
)
RETURNS nvarchar(450)
AS
BEGIN
	DECLARE @ConvertedScopeName nvarchar(450)

	set @ConvertedScopeName = REPLACE(@ScopeName, 
						'.' + CAST(@OldDefinitionID as varchar) + @ScopeDelimiter, 
						'.'+ CAST(@NewDefinitionID as varchar) + @ScopeDelimiter)
	RETURN @ConvertedScopeName
END
GO
PRINT N'Altering Procedure [dbo].[netBlockTypeCheckUsage]...';


GO

ALTER PROCEDURE [dbo].[netBlockTypeCheckUsage]
(
	@BlockTypeID		INT,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE @ContentID INT;
    DECLARE @WorkID INT;

	IF (@OnlyPublished = 1)
	BEGIN
         -- This captures when block has properties that are stored directly on block
        SELECT TOP 1
			@ContentID = tblContentProperty.fkContentID, 
			@WorkID = 0
		FROM 
			tblContentProperty 
		INNER JOIN 
			tblPropertyDefinition ON tblContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        WHERE tblPropertyDefinition.fkContentTypeID = @BlockTypeID

         IF (@ContentID IS NOT NULL)
            BEGIN
                SELECT
                    @ContentID AS ContentID,
                    @WorkID AS WorkID
            END
            ELSE
            BEGIN
                -- This capture the rare case when a block has no values stored on it self but has nested block (list or single)
                SELECT TOP 1
			        Property.fkContentID as ContentID, 
			        0 AS WorkID
		        FROM tblContentProperty as Property WITH(INDEX(IDX_tblContentProperty_ScopeName))
		        INNER JOIN dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON 
			        Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		    END
	END
	ELSE
	BEGIN
        -- This captures when block has properties that are stored directly on block
        SELECT TOP 1
			@ContentID = tblWorkContent.fkContentID, 
			@WorkID = tblWorkContent.pkID
		FROM 
			tblWorkContentProperty 
		INNER JOIN 
			tblPropertyDefinition ON tblWorkContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
			tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
        WHERE tblPropertyDefinition.fkContentTypeID = @BlockTypeID

        IF (@ContentID IS NOT NULL)
        BEGIN
            SELECT
                @ContentID AS ContentID,
                @WorkID AS WorkID
        END
        ELSE
        BEGIN
            -- This capture the rare case when a block has no values stored on it self but has nested block (list or single)
            SELECT TOP 1
			    WorkContent.fkContentID as ContentID, 
			    WorkContent.pkID as WorkID
		    FROM tblWorkContentProperty as Property WITH(INDEX(IDX_tblWorkContentProperty_ScopeName))
		    INNER JOIN dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON 
			    Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		    INNER JOIN
			    tblWorkContent as WorkContent ON Property.fkWorkContentID=WorkContent.pkID
        END
	END
END
GO
PRINT N'Altering Procedure [dbo].[netBlockTypeGetUsage]...';


GO
ALTER PROCEDURE [dbo].[netBlockTypeGetUsage]
(
	@BlockTypeID		INT,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	SET NOCOUNT ON
	
	IF (@OnlyPublished = 1)
	BEGIN
        -- This captures when block has properties that are stored directly on block
         SELECT DISTINCT
			tblContentProperty.fkContentID AS ContentID, 
			0 AS WorkID,
            tblContentLanguage.Name,
			tblLanguageBranch.LanguageID AS LanguageBranch
		FROM 
			tblContentProperty 
		INNER JOIN
            tblPropertyDefinition ON tblContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
            tblContentLanguage ON tblContentProperty.fkContentID = tblContentLanguage.fkContentID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblContentProperty.fkLanguageBranchID
        WHERE
            tblPropertyDefinition.fkContentTypeID = @BlockTypeID
        UNION
        -- This capture the rare case when a block has no values stored on it self but has nested block (list or single)
		SELECT DISTINCT
			Property.fkContentID as ContentID, 
			0 AS WorkID,
			ContentLanguage.Name,
			LanguageBranch.LanguageID AS LanguageBranch
		FROM tblContentProperty as Property WITH(INDEX(IDX_tblContentProperty_ScopeName))
		INNER JOIN
            dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		INNER JOIN
            tblContentLanguage as ContentLanguage ON Property.fkContentID=ContentLanguage.fkContentID
		INNER JOIN
            tblLanguageBranch as LanguageBranch ON LanguageBranch.pkID=ContentLanguage.fkLanguageBranchID
	END
	ELSE
	BEGIN
         -- This captures when block has properties that are stored directly on block
        SELECT DISTINCT
			tblWorkContent.fkContentID AS ContentID, 
			tblWorkContent.pkID AS WorkID,
            tblWorkContent.Name,
			tblLanguageBranch.LanguageID AS LanguageBranch
		FROM 
			tblWorkContentProperty 
		INNER JOIN
            tblPropertyDefinition ON tblWorkContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
            tblWorkContent ON tblWorkContentProperty.fkWorkContentID = tblWorkContent.pkID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblWorkContent.fkLanguageBranchID
        WHERE
            tblPropertyDefinition.fkContentTypeID = @BlockTypeID
        UNION
        -- This capture the rare case when a block has no values stored on it self but has nested block (list or single)
		SELECT DISTINCT
			WorkContent.fkContentID as ContentID, 
			WorkContent.pkID as WorkID,
			WorkContent.Name,
			LanguageBranch.LanguageID AS LanguageBranch
		FROM tblWorkContentProperty as Property WITH(INDEX(IDX_tblWorkContentProperty_ScopeName))
		INNER JOIN
            dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		INNER JOIN
            tblWorkContent as WorkContent ON WorkContent.pkID = Property.fkWorkContentID
		INNER JOIN
            tblLanguageBranch as LanguageBranch ON LanguageBranch.pkID=WorkContent.fkLanguageBranchID
	END
END
GO
PRINT N'Altering Procedure [dbo].[netConvertPropertyForPageType]...';


GO
ALTER PROCEDURE [dbo].[netConvertPropertyForPageType]
(
	@PageID		INT,
	@FromPageType	INT,
	@FromPropertyID 	INT,
	@ToPropertyID		INT,
	@Recursive		BIT,
	@MasterLanguageID INT,
	@IsTest			BIT
)
AS
BEGIN

	DECLARE @cnt INT;
	DECLARE @LanguageSpecific INT
	DECLARE @LanguageSpecificSource INT
	DECLARE @IsBlock BIT
    DECLARE @IsList BIT
    DECLARE @IsContentArea BIT
	SET @LanguageSpecific = 0
	SET @LanguageSpecificSource = 0
	SET @IsBlock = 0
    SET @IsList = 0
    SET @IsContentArea = 0

	CREATE TABLE  #updatepages(fkChildID int)
 
	INSERT INTO #updatepages(fkChildID)  
	SELECT fkChildID 
	FROM tblTree tree
	INNER JOIN tblPage page
	ON page.pkID = tree.fkChildID 
	WHERE @Recursive = 1
	AND tree.fkParentID = @PageID
	AND page.fkPageTypeID = @FromPageType
	UNION (SELECT pkID FROM tblPage WHERE pkID = @PageID AND fkPageTypeID = @FromPageType)

	IF @IsTest = 1
	BEGIN	
		SET @cnt = (	SELECT COUNT(*)
				FROM tblProperty 
				WHERE (fkPageDefinitionID = @FromPropertyID
				or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID))
			+ (	SELECT COUNT(*)
				FROM tblWorkProperty 
				WHERE (fkPageDefinitionID = @FromPropertyID
				or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
				AND EXISTS (
				SELECT * 
					FROM tblWorkPage 
					WHERE pkID = fkWorkPageID
					AND  EXISTS (
						SELECT * from #updatepages WHERE fkChildID=fkPageID)
				))
		IF @ToPropertyID IS NULL OR @ToPropertyID = 0-- mark deleted rows with -
			SET @cnt = -@cnt
	END
	ELSE
	BEGIN
		IF @ToPropertyID IS NULL OR @ToPropertyID = 0-- no definition exists for the new page type for this property so remove it
		BEGIN
			DELETE
			FROM tblProperty 
			WHERE (fkPageDefinitionID = @FromPropertyID
			or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
			AND  EXISTS (
				SELECT * from #updatepages WHERE fkChildID=fkPageID)

			SET @cnt = -@@rowcount

			DELETE 
			FROM tblWorkProperty 
			WHERE (fkPageDefinitionID = @FromPropertyID
			or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
			AND EXISTS (
				SELECT * 
				FROM tblWorkPage 
				WHERE pkID = fkWorkPageID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)
				)
			SET @cnt = @cnt-@@rowcount 
		END 	
		ELSE IF @FromPropertyID IS NOT NULL -- from property exists and has to be replaced
		BEGIN
			-- Need to check if the property we're converting to is unique for each language or not
			SELECT @LanguageSpecific = LanguageSpecific 
			FROM tblPageDefinition 
			WHERE pkID = @ToPropertyID

			-- Need to check if the property we're converting from is unique for each language or not
			SELECT @LanguageSpecificSource = LanguageSpecific 
			FROM tblPageDefinition 
			WHERE pkID = @FromPropertyID
			
			-- Need to check if the property we're converting is a block (Property 12 is a block)
			SELECT @IsBlock = CAST(count(*) as bit)
			FROM tblPageDefinition 
			Where pkID = @FromPropertyID and Property = 12

            -- Need to check if the property we're converting is a list
			SELECT @IsList = CAST(count(*) as bit)
			FROM tblPageDefinition 
			Where pkID = @FromPropertyID and IsList = 1

             -- Need to check if the property we're converting is a ContentArea
			SELECT @IsContentArea = CAST(count(*) as bit)
			FROM tblPageDefinition INNER JOIN tblPageDefinitionType on tblPageDefinition.fkPageDefinitionTypeID = tblPageDefinitionType.pkID
			WHERE tblPageDefinition.pkID = @FromPropertyID AND tblPageDefinitionType.TypeName = 'EPiServer.SpecializedProperties.PropertyContentArea'

            DECLARE @DefinitionTypeFrom int
		    DECLARE @DefinitionTypeTo int

            IF @IsList = 1
			BEGIN
				SET @DefinitionTypeFrom = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@FromPropertyID)
				SET @DefinitionTypeTo = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@ToPropertyID)
				IF (@DefinitionTypeFrom <> @DefinitionTypeTo)
				BEGIN
					RAISERROR (N'Property definitions are not of same list type.', 16, 1)
					RETURN 0
				END
				
				-- Update older versions of block
				-- update scopename in tblWorkProperty
                 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, '(')
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '(%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 SET @cnt = @@rowcount

				-- Update current version of block
				-- update scopename in tblProperty
				
                 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, '(')
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '(%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				 SET @cnt = @cnt + @@rowcount
			END

            IF @IsContentArea = 1
			BEGIN

				-- Update older versions of contentarea
				-- update scopename in tblWorkProperty
                 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, ':')
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + ':%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 SET @cnt = @@rowcount

				-- Update current version of contentarea
				-- update scopename in tblProperty
				
                 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, ':')
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + ':%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				 SET @cnt = @cnt + @@rowcount
			END

			IF @IsBlock = 1
			BEGIN
				SET @DefinitionTypeFrom = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@FromPropertyID)
				SET @DefinitionTypeTo = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@ToPropertyID)
				IF (@DefinitionTypeFrom <> @DefinitionTypeTo)
				BEGIN
					RAISERROR (N'Property definitions are not of same block type.', 16, 1)
					RETURN 0
				END
				
				-- Update older versions of block
				-- update scopename in tblWorkProperty
				
				 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
			
				 SET @cnt = @@rowcount

				-- Update current version of block
				-- update scopename in tblProperty
				
				 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt + @@rowcount

                 --Update category tables in case block has category properties
                 UPDATE tblWorkContentCategory
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 FROM tblWorkContentCategory prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkContentID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 UPDATE tblContentCategory 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
			END
			ELSE -- Not a block
			BEGIN
				-- Update older versions
				UPDATE tblWorkProperty SET fkPageDefinitionID = @ToPropertyID
					FROM tblWorkProperty prop
					INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
					WHERE prop.fkPageDefinitionID = @FromPropertyID
					AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
			
				SET @cnt = @@rowcount

				-- Update current version 
				UPDATE tblProperty SET fkPageDefinitionID = @ToPropertyID
				WHERE fkPageDefinitionID = @FromPropertyID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt + @@rowcount
			END

			IF (@LanguageSpecific < 3 AND @LanguageSpecificSource > 2)
			BEGIN
				-- The destination property is not language specific which means
				-- that we need to remove all of the old properties in other
				-- languages that could not be mapped
				DELETE FROM tblWorkProperty
					FROM tblWorkProperty prop
					INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
					WHERE (prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
					or prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%')
					AND wpa.fkLanguageBranchID <> @MasterLanguageID
					AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
				
				SET @cnt = @cnt - @@rowcount		
				
				DELETE FROM tblProperty 
				WHERE (fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				or ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%')
				AND fkLanguageBranchID <> @MasterLanguageID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt - @@rowcount				
			END
			ELSE IF (@LanguageSpecificSource < 3)
			BEGIN
				-- Converting from language neutral to language supporting property
				-- We must copy existing master language property to other languages for the page
				
				-- NOTE: Due to the way language neutral properties are loaded, that is they are always
				-- loaded from published version, see netPageDataLoadVersion it is sufficient to add property
				-- values to tblProperty (no need to update tblWorkProperty
				
				INSERT INTO tblProperty
					(fkPageDefinitionID,
					fkPageID,
					fkLanguageBranchID,
					ScopeName,
                    ListIndex,
					Boolean,
					Number,
					FloatNumber,
					PageType,
					PageLink,
					LinkGuid,
					Date,
					String,
					LongString,
					LongStringLength)
				SELECT 
					CASE @IsBlock when 1 then Prop.fkPageDefinitionID else @ToPropertyID end, 
					Prop.fkPageID,
					Lang.fkLanguageBranchID,
					Prop.ScopeName,
                    Prop.ListIndex,
					Prop.Boolean,
					Prop.Number,
					Prop.FloatNumber,
					Prop.PageType,
					Prop.PageLink,
					Prop.LinkGuid,
					Prop.Date,
					Prop.String,
					Prop.LongString,
					Prop.LongStringLength
				FROM
				tblPageLanguage Lang
				INNER JOIN
				tblProperty Prop ON Prop.fkLanguageBranchID = @MasterLanguageID
				WHERE
				Prop.fkPageID = @PageID AND
				(Prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				or Prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%') AND
				Prop.fkLanguageBranchID = @MasterLanguageID AND
				Lang.fkLanguageBranchID <> @MasterLanguageID AND
				Lang.fkPageID = @PageID

				-- Need to add entries to tblWorkProperty for all pages not in the master language
				-- First we need to read the master language property into a temp table
				CREATE TABLE #TempWorkProperty
				(
					fkPageDefinitionID int,
					ScopeName nvarchar(450),
                    ListIndex int,
					Boolean bit,
					Number int,
					FloatNumber float,
					PageType int,
					PageLink int,
				    LinkGuid uniqueidentifier,
					Date datetime,
					String nvarchar(450),
					LongString nvarchar(max)
				)

				INSERT INTO #TempWorkProperty
				SELECT
					Prop.fkPageDefinitionID,
					Prop.ScopeName,
                    Prop.ListIndex,
					Prop.Boolean,
					Prop.Number,
					Prop.FloatNumber,
					Prop.PageType,
					Prop.PageLink,
				    Prop.LinkGuid,
					Prop.Date,
					Prop.String,
					Prop.LongString
				FROM
					tblWorkProperty AS Prop
					INNER JOIN
					tblWorkPage AS Page ON Prop.fkWorkPageID = Page.pkID
				WHERE
					(Prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				    or Prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%') AND
					Page.fkLanguageBranchID = @MasterLanguageID AND
					Page.fkPageID = @PageID
					ORDER BY Page.pkID DESC

				-- Now add a new property for every language (except master) using the master value
				INSERT INTO	tblWorkProperty 
				SELECT
					CASE @IsBlock when 1 then TempProp.fkPageDefinitionID else @ToPropertyID end,
					Page.pkID,
					TempProp.ScopeName,
                    TempProp.ListIndex,
					TempProp.Boolean,
					TempProp.Number,
					TempProp.FloatNumber,
					TempProp.PageType,
					TempProp.PageLink,
					TempProp.Date,
					TempProp.String,
					TempProp.LongString,
					TempProp.LinkGuid
				FROM 
					tblWorkPage AS Page
					CROSS JOIN #TempWorkProperty AS TempProp
				WHERE
                    Page.fkPageID = @PageID AND
					Page.fkLanguageBranchID <> @MasterLanguageID

				DROP TABLE #TempWorkProperty

			END
		END
	END

	DROP TABLE #updatepages

	RETURN (@cnt)
END
GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';


GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8012
GO
PRINT N'Creating Procedure [dbo].[netContentLoadScopedProperties]...';


GO
CREATE PROCEDURE dbo.netContentLoadScopedProperties
(
    @ContentID INT,
	@LanguageBranchID INT,
    @ScopeName NVARCHAR(450),
    @PropertyDefinitionID INT = NULL
)
AS
BEGIN
	SELECT
		tblContentProperty.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblPropertyDefinition.Name AS PropertyName,
		tblPropertyDefinition.pkID as PropertyDefinitionID,
		ScopeName,
		CONVERT(INT, Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		tblContentProperty.ContentType AS PageType,
		ContentLink,
		LinkGuid,	
		Date AS DateValue,
		String,
		LongString,
		tblContentProperty.fkLanguageBranchID AS PageLanguageBranchID,
		NULL AS Guid,
        0 AS DelayedList
	FROM tblContentProperty
	INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentProperty.fkPropertyDefinitionID
	WHERE tblContentProperty.fkContentID = @ContentID
    AND tblContentProperty.fkPropertyDefinitionID = @PropertyDefinitionID
    AND NOT tblPropertyDefinition.fkContentTypeID IS NULL AND
		(tblContentProperty.fkLanguageBranchID = @LanguageBranchID
	OR
		tblContentProperty.BranchSpecificScope = 0)
    UNION
    SELECT
		tblContentProperty.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblPropertyDefinition.Name AS PropertyName,
		tblPropertyDefinition.pkID as PropertyDefinitionID,
		ScopeName,
		CONVERT(INT, Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		tblContentProperty.ContentType AS PageType,
		ContentLink,
		LinkGuid,	
		Date AS DateValue,
		String,
		LongString,
		tblContentProperty.fkLanguageBranchID AS PageLanguageBranchID,
		NULL AS Guid,
        0 AS DelayedList
	FROM tblContentProperty
	INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentProperty.fkPropertyDefinitionID
	WHERE tblContentProperty.fkContentID = @ContentID
    AND ScopeName LIKE @ScopeName + '%'
    AND NOT tblPropertyDefinition.fkContentTypeID IS NULL AND
		(tblContentProperty.fkLanguageBranchID = @LanguageBranchID
	OR
		tblContentProperty.BranchSpecificScope = 0)
END

PRINT N'Altering Procedure [dbo].[netVersionFilterList]...';
GO


ALTER PROCEDURE [dbo].[netVersionFilterList]
(
	@StartIndex INT,
	@MaxRows INT,
	@ContentID INT = NULL,
	@ChangedBy NVARCHAR(255) = NULL,
	@ExcludeDeleted BIT = 0,
	@LanguageIds dbo.IDTable READONLY,
	@Statuses dbo.IDTable READONLY,
	@SortColumn varchar(40) = 'Saved',
	@SortDirection bit = 1,
	@Name NVARCHAR(255) = NULL,
    @StartDelayPublishUntil DATETIME = NULL,
    @EndDelayPublishUntil DATETIME = NULL
)
AS

BEGIN	
	SET NOCOUNT ON

    DECLARE @OrderBy NVARCHAR(64)

	DECLARE @StatusCount INT
	SELECT @StatusCount = COUNT(*) FROM @Statuses

	DECLARE @LanguageCount INT
	SELECT @LanguageCount = COUNT(*) FROM @LanguageIds;
	--Optimized for contentid with and without language, those are most used
	IF (@ContentID IS NOT NULL AND @ChangedBy IS NULL AND @ExcludeDeleted = 0 AND @StatusCount = 0 AND @Name IS NULL AND @StartDelayPublishUntil IS NULL AND @EndDelayPublishUntil IS NULL)
		IF (@LanguageCount = 0)
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID
			)
			SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, (SELECT COUNT(*) FROM TempResult) AS 'TotalRows'
				FROM    TempResult
				WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
		END
		ELSE
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID AND
				    W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)

		)
		SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, (SELECT COUNT(*) FROM TempResult) AS 'TotalRows'
				FROM    TempResult
				WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
		END
	ELSE
		BEGIN
        	SET @OrderBy =
		        CASE @SortColumn
			        WHEN 'Saved' THEN 'W.Saved'
			        WHEN 'Name' THEN 'W.Name'
			        WHEN 'ID' THEN 'W.fkContentID'
                    WHEN 'DelayPublishUntil' THEN 'W.DelayPublishUntil'
                    WHEN 'SavedBy' THEN 'W.ChangedByName'
                    WHEN 'StatusChangedBy' THEN 'W.NewStatusByName'
                    
		        END
            IF(@SortDirection = 1)
		        SET @OrderBy = @OrderBy + ' DESC'

            DECLARE @SqlQuery NVARCHAR(MAX) = '
		    WITH TempResult as
		    (
			    SELECT ROW_NUMBER() OVER(ORDER BY '+ @OrderBy +' ) as RowNumber,
				    W.fkContentID AS ContentID,
				    W.pkID AS WorkID,
				    W.Status AS VersionStatus,
				    W.ChangedByName AS SavedBy,
				    W.Saved AS ItemCreated,
				    W.Name,
				    W.fkLanguageBranchID as LanguageBranchID,
				    W.CommonDraft,
				    W.fkMasterVersionID as MasterVersion,
				    CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
				    W.NewStatusByName As StatusChangedBy,
				    W.DelayPublishUntil
			    FROM
				    tblWorkContent AS W
				    INNER JOIN
				    tblContent AS C ON C.pkID=W.fkContentID
			    WHERE' + CHAR(13);
        END

		DECLARE @IncludeAnd BIT = 0
		IF @ContentID IS NOT NULL
		BEGIN
			SET @SqlQuery += 'W.fkContentID=@ContentID' + CHAR(13);
			SET @IncludeAnd = 1
		END
		IF @ChangedBy IS NOT NULL
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.ChangedByName=@ChangedBy' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.ChangedByName=@ChangedBy' + CHAR(13);
			SET @IncludeAnd = 1
		END
		IF @Name IS NOT NULL
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.Name LIKE @Name' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.Name LIKE @Name' + CHAR(13);
			SET @IncludeAnd = 1
		END
		IF @StatusCount > 0
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
			SET @IncludeAnd = 1
		END
		IF @LanguageCount > 0
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
			SET @IncludeAnd = 1
		END
		IF @ExcludeDeleted = 1
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND C.Deleted = 0' + CHAR(13);
			ELSE
				SET @SqlQuery += 'C.Deleted = 0' + CHAR(13);
		END

		IF @StartDelayPublishUntil IS NOT NULL 
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
		END

        IF @EndDelayPublishUntil IS NOT NULL 
		BEGIN
			IF @IncludeAnd = 1
				SET @SqlQuery += 'AND W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
			ELSE
				SET @SqlQuery += 'W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
		END


		SET @SqlQuery += ')' + CHAR(13);
		SET @SqlQuery += 'SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, (SELECT COUNT(*) FROM TempResult) AS ''TotalRows''
			FROM    TempResult
			WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)'
   		
		print @SqlQuery
		EXEC sp_executesql @SqlQuery, N'@StartIndex INT, @MaxRows INT, @ContentID INT, @ChangedBy NVARCHAR(255), @Name NVARCHAR(255), @Statuses dbo.IDTable READONLY, @LanguageIds dbo.IDTable READONLY, @ExcludeDeleted BIT, @StartDelayPublishUntil DATETIME, @EndDelayPublishUntil DATETIME', 
			@StartIndex = @StartIndex, @MaxRows = @MaxRows, @ContentID = @ContentID, @ChangedBy = @ChangedBy, @Name = @Name, @Statuses = @Statuses, @LanguageIds = @LanguageIds, @ExcludeDeleted = @ExcludeDeleted, @StartDelayPublishUntil = @StartDelayPublishUntil, @EndDelayPublishUntil = @EndDelayPublishUntil
END
GO

CREATE NONCLUSTERED INDEX [IDX_tblWorkContent_Name] ON [dbo].[tblWorkContent]
(
    [Name] ASC
)
GO

CREATE NONCLUSTERED INDEX [IDX_tblWorkContent_DelayPublishUntil] ON [dbo].[tblWorkContent]
(
    [DelayPublishUntil] DESC
)
GO



GO
PRINT N'Update complete.';


GO
