--beginvalidatingquery
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
begin
	declare @ver int
	exec @ver=sp_DatabaseVersion
	if (@ver >= 8013)
		select 0, 'Already correct database version'
	else if (@ver = 8012)
		select 1, 'Upgrading database'
	else
		select -1, 'Invalid database version detected'
end
else
	select -1, 'Not an EPiServer database'
--endvalidatingquery

GO

PRINT N'Altering Table [dbo].[tblContentSoftlink]...';
GO

ALTER TABLE [dbo].[tblContentSoftlink]
    ADD [fkOwnerPropertyDefinitionID] INT NULL;


GO
PRINT N'Creating Foreign Key [dbo].[FK_tblContentSoftlink_tblPropertyDefinition]...';


GO
ALTER TABLE [dbo].[tblContentSoftlink] WITH NOCHECK
    ADD CONSTRAINT [FK_tblContentSoftlink_tblPropertyDefinition] FOREIGN KEY ([fkOwnerPropertyDefinitionID]) REFERENCES [dbo].[tblPropertyDefinition] ([pkID]);


GO
PRINT N'Altering Procedure [dbo].[netSoftLinkByExternalLink]...';


GO
ALTER PROCEDURE [dbo].[netSoftLinkByExternalLink]
(
	@ContentLink NVARCHAR(255),
	@ContentGuid uniqueidentifier = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT 
		pkID,
		fkOwnerContentID AS OwnerID,
		fkReferencedContentGUID AS ReferencedGUID,
		NULL AS OwnerName,
		NULL AS ReferencedName,
		OwnerLanguageID,
		ReferencedLanguageID,
		LinkURL,
		LinkType as ReferenceType ,
		LinkProtocol,
		LastCheckedDate,
		FirstDateBroken,
		HttpStatusCode,
		LinkStatus,
        fkOwnerPropertyDefinitionID  AS OwnerPropertyDefinitionID
	FROM tblContentSoftlink 
	WHERE [fkReferencedContentGUID] = @ContentGuid OR [ContentLink] = @ContentLink
END
GO
PRINT N'Altering Procedure [dbo].[netSoftLinkByPageLink]...';


GO
ALTER PROCEDURE [dbo].[netSoftLinkByPageLink]
(
	@PageLink NVARCHAR(255)
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT 
		pkID,
		fkOwnerContentID AS OwnerID,
		fkReferencedContentGUID AS ReferencedGUID,
		NULL AS OwnerName,
		NULL AS ReferencedName,
		OwnerLanguageID,
		ReferencedLanguageID,
		LinkURL,
		LinkType as ReferenceType ,
		LinkProtocol,
		LastCheckedDate,
		FirstDateBroken,
		HttpStatusCode,
		LinkStatus,
        fkOwnerPropertyDefinitionID  AS OwnerPropertyDefinitionID
	FROM tblContentSoftlink 
	WHERE [ContentLink] = @PageLink
END
GO
PRINT N'Altering Procedure [dbo].[netSoftLinkByUrl]...';


GO
ALTER PROCEDURE dbo.netSoftLinkByUrl
(
	@LinkURL NVARCHAR(2048),
	@ExactMatch INT = 1
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT 
		pkID,
		fkOwnerContentID AS OwnerContentID,
		fkReferencedContentGUID AS ReferencedContentGUID,
		OwnerLanguageID,
		ReferencedLanguageID,
		LinkURL,
		LinkType,
		LinkProtocol,
		LastCheckedDate,
		FirstDateBroken,
		HttpStatusCode,
		LinkStatus,
        fkOwnerPropertyDefinitionID  AS OwnerPropertyDefinitionID
	FROM tblContentSoftlink 
	WHERE (@ExactMatch=1 AND LinkURL LIKE @LinkURL) OR (@ExactMatch=0 AND LinkURL LIKE (@LinkURL + '%'))
END
GO
PRINT N'Altering Procedure [dbo].[netSoftLinkInsert]...';


GO
ALTER PROCEDURE dbo.netSoftLinkInsert
(
	@OwnerContentID	INT,
	@ReferencedContentGUID uniqueidentifier,
	@LinkURL	NVARCHAR(2048),
	@LinkType	INT,
	@LinkProtocol	NVARCHAR(10),
	@ContentLink	NVARCHAR(255),
	@LastCheckedDate datetime,
	@FirstDateBroken datetime,
	@HttpStatusCode int,
	@LinkStatus int,
	@OwnerLanguageID int,
	@ReferencedLanguageID int,
    @OwnerPropertyDefinitionID int
)
AS
BEGIN
	INSERT INTO tblContentSoftlink
		(fkOwnerContentID,
		fkReferencedContentGUID,
	    OwnerLanguageID,
		ReferencedLanguageID,
		LinkURL,
		LinkType,
		LinkProtocol,
		ContentLink,
		LastCheckedDate,
		FirstDateBroken,
		HttpStatusCode,
		LinkStatus,
        fkOwnerPropertyDefinitionID)
	VALUES
		(@OwnerContentID,
		@ReferencedContentGUID,
		@OwnerLanguageID,
		@ReferencedLanguageID,
		@LinkURL,
		@LinkType,
		@LinkProtocol,
		@ContentLink,
		@LastCheckedDate,
		@FirstDateBroken,
		@HttpStatusCode,
		@LinkStatus,
        @OwnerPropertyDefinitionID)
END
GO
PRINT N'Altering Procedure [dbo].[netSoftLinkList]...';


GO
ALTER PROCEDURE dbo.netSoftLinkList
(
	@ReferenceGUID	uniqueidentifier,
	@Reversed INT = 0
)
AS
BEGIN
	SET NOCOUNT ON
	
	IF @Reversed = 1
		SELECT 
			pkID,
			fkOwnerContentID AS OwnerContentID,
			fkReferencedContentGUID AS ReferencedContentGUID,
			OwnerLanguageID,
			ReferencedLanguageID,
			LinkURL,
			LinkType,
			LinkProtocol,
			LastCheckedDate,
			FirstDateBroken,
			HttpStatusCode,
			LinkStatus,
            fkOwnerPropertyDefinitionID  AS OwnerPropertyDefinitionID
		FROM tblContentSoftlink 
		WHERE fkReferencedContentGUID=@ReferenceGUID
	ELSE
		SELECT 
			SoftLink.pkID,
			Content.pkID AS OwnerContentID,
			SoftLink.fkReferencedContentGUID AS ReferencedContentGUID,
			SoftLink.OwnerLanguageID,
			SoftLink.ReferencedLanguageID,
			SoftLink.LinkURL,
			SoftLink.LinkType,
			SoftLink.LinkProtocol,
			SoftLink.LastCheckedDate,
			SoftLink.FirstDateBroken,
			SoftLink.HttpStatusCode,
			SoftLink.LinkStatus,
            SoftLink.fkOwnerPropertyDefinitionID AS OwnerPropertyDefinitionID
		FROM tblContentSoftlink AS SoftLink
		INNER JOIN tblContent as Content ON SoftLink.fkOwnerContentID = Content.pkID
		WHERE Content.ContentGUID=@ReferenceGUID
END
GO
PRINT N'Altering Procedure [dbo].[netSoftLinksGetBroken]...';


GO
ALTER PROCEDURE [dbo].[netSoftLinksGetBroken]
	@SkipCount int,
	@MaxResults int,
	@RootPageId int
AS
BEGIN
	SELECT [pkID]
		,[fkOwnerContentID]
		,[fkReferencedContentGUID]
		,[OwnerLanguageID]
		,[ReferencedLanguageID]
		,[LinkURL]
		,[LinkType]
		,[LinkProtocol]
		,[ContentLink]
		,[LastCheckedDate]
		,[FirstDateBroken]
		,[HttpStatusCode]
		,[LinkStatus]
        ,[fkOwnerPropertyDefinitionID]
	FROM (
		SELECT [pkID]
			,[fkOwnerContentID]
			,[fkReferencedContentGUID]
			,[OwnerLanguageID]
			,[ReferencedLanguageID]
			,[LinkURL]
			,[LinkType]
			,[LinkProtocol]
			,[ContentLink]
			,[LastCheckedDate]
			,[FirstDateBroken]
			,[HttpStatusCode]
			,[LinkStatus]
            ,[fkOwnerPropertyDefinitionID]
			,ROW_NUMBER() OVER (ORDER BY pkID ASC) as RowNumber
		FROM [tblContentSoftlink]
		INNER JOIN tblTree ON tblContentSoftlink.fkOwnerContentID = tblTree.fkChildID 
		WHERE (tblTree.fkParentID = @RootPageId OR (tblContentSoftlink.fkOwnerContentID = @RootPageId AND tblTree.NestingLevel = 1)) AND LinkStatus <> 0
		) BrokenLinks
	WHERE BrokenLinks.RowNumber > @SkipCount AND BrokenLinks.RowNumber <= @SkipCount+@MaxResults
END
GO
PRINT N'Altering Procedure [dbo].[netPropertyDefinitionDelete]...';


GO
ALTER PROCEDURE [dbo].[netPropertyDefinitionDelete]
(
	@PropertyDefinitionID INT
)
AS
BEGIN
	DECLARE @ScopedProperties TABLE (ScopeName nvarchar(450))
	INSERT INTO @ScopedProperties SELECT * FROM dbo.GetExistingScopesForDefinition(@PropertyDefinitionID) 
	DELETE FROM tblContentProperty WHERE ScopeName IN (SELECT ScopeName FROM @ScopedProperties) 
	DELETE FROM tblWorkContentProperty WHERE ScopeName IN (SELECT ScopeName FROM @ScopedProperties)
	DELETE FROM tblPropertyDefault WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblProperty WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblWorkProperty WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblCategoryPage WHERE CategoryType=@PropertyDefinitionID
	DELETE FROM tblWorkCategory WHERE CategoryType=@PropertyDefinitionID
    DELETE FROM tblContentSoftlink WHERE fkOwnerPropertyDefinitionID=@PropertyDefinitionID
	DELETE FROM tblPageDefinition WHERE pkID=@PropertyDefinitionID
END
GO

PRINT N'Altering Procedure [dbo].[editSaveContentVersionData]...';
GO


ALTER PROCEDURE [dbo].[editSaveContentVersionData]
(
	@WorkContentID		INT,
	@UserName			NVARCHAR(255),
	@Saved				DATETIME,
	@Name				NVARCHAR(255)		= NULL,
	@ExternalURL		NVARCHAR(255)		= NULL,
	@Created			DATETIME			= NULL,
	@Changed			BIT					= 0,
	@StartPublish		DATETIME			= NULL,
	@StopPublish		DATETIME			= NULL,
	@ChildOrder			INT					= 3,
	@PeerOrder			INT					= 100,
	@ContentLinkGUID	UNIQUEIDENTIFIER	= NULL,
	@LinkURL			NVARCHAR(255)		= NULL,
	@BlobUri			NVARCHAR(255)		= NULL,
	@ThumbnailUri		NVARCHAR(255)		= NULL,
	@LinkType			INT					= 0,
	@FrameID			INT					= NULL,
	@VisibleInMenu		BIT					= NULL,
	@ArchiveContentGUID	UNIQUEIDENTIFIER	= NULL,
	@ContentAssetsID	UNIQUEIDENTIFIER	= NULL,
	@ContentOwnerID		UNIQUEIDENTIFIER	= NULL,
	@URLSegment			NVARCHAR(255)		= NULL,
	@SetStartPublish    BIT					= 1
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	DECLARE @ChangedDate			DATETIME
	DECLARE @ContentID				INT
	DECLARE @ContentTypeID			INT
	DECLARE @ParentID				INT
	DECLARE @AssetsID				UNIQUEIDENTIFIER
	DECLARE @OwnerID				UNIQUEIDENTIFIER
	DECLARE @CurrentLangBranchID	INT
	DECLARE @IsMasterLang			BIT
	
	/* Pull some useful information from the published Content */
	SELECT
		@ContentID				= fkContentID,
		@ParentID				= fkParentID,
		@ContentTypeID			= fkContentTypeID,
		@AssetsID				= ContentAssetsID,
		@OwnerID				= ContentOwnerID,
		@IsMasterLang			= CASE WHEN tblContent.fkMasterLanguageBranchID=tblWorkContent.fkLanguageBranchID THEN 1 ELSE 0 END,
		@CurrentLangBranchID	= fkLanguageBranchID
	FROM
		tblWorkContent
	INNER JOIN tblContent ON tblContent.pkID=tblWorkContent.fkContentID
	INNER JOIN tblContentType ON tblContentType.pkID=tblContent.fkContentTypeID
	WHERE
		tblWorkContent.pkID=@WorkContentID
	
	if (@ContentID IS NULL)
	BEGIN
		RAISERROR (N'editSaveContentVersionData: The WorkContentId dosen´t exist (WorkContentID=%d)', 16, 1, @WorkContentID)
		RETURN -1
	END			
		IF ((@AssetsID IS NULL) AND (@ContentAssetsID IS NOT NULL))
		BEGIN
			UPDATE
				tblContent
			SET
				ContentAssetsID = @ContentAssetsID
			WHERE
				pkID=@ContentID
		END

		IF ((@OwnerID IS NULL) AND (@ContentOwnerID IS NOT NULL))
		BEGIN
			UPDATE
				tblContent
			SET
				ContentOwnerID = @ContentOwnerID
			WHERE
				pkID=@ContentID
		END

		/* Set new values for work Content */
		UPDATE
			tblWorkContent
		SET
			ChangedByName		= @UserName,
			ContentLinkGUID		= @ContentLinkGUID,
			ArchiveContentGUID	= @ArchiveContentGUID,
			fkFrameID			= @FrameID,
			Name				= @Name,
			LinkURL				= @LinkURL,
			BlobUri				= @BlobUri,
			ThumbnailUri		= @ThumbnailUri,
			ExternalURL			= @ExternalURL,
			URLSegment			= @URLSegment,
			VisibleInMenu		= @VisibleInMenu,
			LinkType			= @LinkType,
			Created				= COALESCE(@Created, Created),
			Saved				= @Saved,
			StartPublish		= CASE WHEN @SetStartPublish = 1 THEN @StartPublish ELSE StartPublish END,
			StopPublish			= @StopPublish,
			ChildOrderRule		= @ChildOrder,
			PeerOrder			= COALESCE(@PeerOrder, PeerOrder),
			ChangedOnPublish	= @Changed
		WHERE
			pkID=@WorkContentID
		
	IF EXISTS(SELECT * FROM tblContentLanguage WHERE fkContentID=@ContentID AND fkLanguageBranchID=@CurrentLangBranchID AND Status <> 4)
	BEGIN

			UPDATE
				tblContentLanguage
			SET
				Name			= @Name,
				Created			= @Created,
				Saved			= @Saved,
				URLSegment		= @URLSegment,
				LinkURL			= @LinkURL,
				BlobUri			= @BlobUri,
				ThumbnailUri	= @ThumbnailUri,
				StartPublish	= CASE WHEN @SetStartPublish = 1 THEN @StartPublish ELSE StartPublish END,
				StopPublish		= @StopPublish,
				ExternalURL		= Lower(@ExternalURL),
				fkFrameID		= @FrameID,
				AutomaticLink	= CASE WHEN @LinkType = 2 OR @LinkType = 3 THEN 0 ELSE 1 END,
				FetchData		= CASE WHEN @LinkType = 4 THEN 1 ELSE 0 END
			WHERE
				fkContentID=@ContentID AND fkLanguageBranchID=@CurrentLangBranchID

			/* Set some values needed for proper display in edit tree even though we have not yet published the Content */
			IF @IsMasterLang = 1
			BEGIN
				UPDATE
					tblContent
				SET
					ArchiveContentGUID	= @ArchiveContentGUID,
					ChildOrderRule		= @ChildOrder,
					PeerOrder			= @PeerOrder,
					VisibleInMenu		= @VisibleInMenu
				WHERE
					pkID=@ContentID 
			END

	END

    /*If the matser language we are updating*/
    IF @IsMasterLang = 1
    BEGIN
        /* If there is no published on master language branch for the content*/
        IF NOT EXISTS(
            SELECT *
            FROM tblContent
            INNER JOIN tblContentLanguage ON tblContent.pkID = tblContentLanguage.fkContentID
            WHERE tblContent.pkID = @ContentID AND tblContent.fkMasterLanguageBranchID = @CurrentLangBranchID AND tblContentLanguage.Status = 4)

        BEGIN
            UPDATE
                tblWorkContent
            SET
                ArchiveContentGUID    = @ArchiveContentGUID,
                ChildOrderRule        = @ChildOrder,
                PeerOrder            = COALESCE(@PeerOrder, PeerOrder),
                VisibleInMenu        = @VisibleInMenu
            WHERE
                tblWorkContent.fkContentID = @ContentID AND tblWorkContent.fkLanguageBranchID != @CurrentLangBranchID AND tblWorkContent.Status <> 4 AND tblWorkContent.Status <> 5
        END
    END
END
GO


PRINT N'Altering Procedure [dbo].[editPublishContentVersion]...';
GO

ALTER PROCEDURE dbo.editPublishContentVersion
(
	@WorkContentID	INT,
	@UserName NVARCHAR(255),
	@TrimVersions BIT = 0,
	@ResetCommonDraft BIT = 1,
	@PublishedDate DATETIME = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	DECLARE @ContentID INT
	DECLARE @retval INT
	DECLARE @FirstPublish BIT
	DECLARE @ParentID INT
	DECLARE @LangBranchID INT
	DECLARE @IsMasterLang BIT
	
	/* Verify that we have a Content to publish */
	SELECT	@ContentID=fkContentID,
			@LangBranchID=fkLanguageBranchID,
			@IsMasterLang = CASE WHEN tblWorkContent.fkLanguageBranchID=tblContent.fkMasterLanguageBranchID THEN 1 ELSE 0 END
	FROM tblWorkContent WITH (ROWLOCK,XLOCK)
	INNER JOIN tblContent WITH (ROWLOCK,XLOCK) ON tblContent.pkID=tblWorkContent.fkContentID
	WHERE tblWorkContent.pkID=@WorkContentID
	
	IF (@@ROWCOUNT <> 1)
		RETURN 0

	IF @PublishedDate IS NULL
		SET @PublishedDate = GetDate()
					
	/* Move Content information from worktable to published table */
	IF @IsMasterLang=1
	BEGIN
		UPDATE 
			tblContent
		SET
			ArchiveContentGUID	= W.ArchiveContentGUID,
			VisibleInMenu	= W.VisibleInMenu,
			ChildOrderRule	= W.ChildOrderRule,
			PeerOrder		= W.PeerOrder
		FROM 
			tblWorkContent AS W
		WHERE 
			tblContent.pkID=W.fkContentID AND 
			W.pkID=@WorkContentID
	END
	
	UPDATE 
			tblContentLanguage WITH (ROWLOCK,XLOCK)
		SET
			ChangedByName	= W.ChangedByName,
			ContentLinkGUID	= W.ContentLinkGUID,
			fkFrameID		= W.fkFrameID,
			Name			= W.Name,
			URLSegment		= W.URLSegment,
			LinkURL			= W.LinkURL,
			BlobUri			= W.BlobUri,
			ThumbnailUri	= W.ThumbnailUri,
			ExternalURL		= Lower(W.ExternalURL),
			AutomaticLink	= CASE WHEN W.LinkType = 2 OR W.LinkType = 3 THEN 0 ELSE 1 END,
			FetchData		= CASE WHEN W.LinkType = 4 THEN 1 ELSE 0 END,
			Created			= W.Created,
			Changed			= CASE WHEN W.ChangedOnPublish=0 AND tblContentLanguage.Status = 4 THEN Changed ELSE @PublishedDate END,
			Saved			= @PublishedDate,
			StartPublish	= COALESCE(W.StartPublish, @PublishedDate),
			StopPublish		= W.StopPublish,
			Status			= 4,
			Version			= @WorkContentID,
			DelayPublishUntil = NULL
		FROM 
			tblWorkContent AS W
		WHERE 
			tblContentLanguage.fkContentID=W.fkContentID AND
			W.fkLanguageBranchID=tblContentLanguage.fkLanguageBranchID AND
			W.pkID=@WorkContentID

	IF @@ROWCOUNT!=1
		RAISERROR (N'editPublishContentVersion: Cannot find correct version in tblContentLanguage for version %d', 16, 1, @WorkContentID)

	/*Set current published version on this language to HasBeenPublished*/
	UPDATE
		tblWorkContent
	SET
		Status = 5
	WHERE
		fkContentID = @ContentID AND
		fkLanguageBranchID = @LangBranchID AND 
		Status = 4 AND
		pkID<>@WorkContentID

	/* Remember that this version has been published, and clear the delay publish date if used */
	UPDATE
		tblWorkContent
	SET
		Status = 4,
		ChangedOnPublish = 0,
		Saved=@PublishedDate,
        ChangedByName=@UserName,
		NewStatusByName=@UserName,
		fkMasterVersionID = NULL,
		DelayPublishUntil = NULL,
		StartPublish = COALESCE(StartPublish, @PublishedDate)
	WHERE
		pkID=@WorkContentID
		
	/* Remove all properties defined for this Content except dynamic properties */
	DELETE FROM 
		tblContentProperty
	FROM 
		tblContentProperty
	INNER JOIN
		tblPropertyDefinition ON fkPropertyDefinitionID=tblPropertyDefinition.pkID
	WHERE 
		fkContentID=@ContentID AND
		fkContentTypeID IS NOT NULL AND
		fkLanguageBranchID=@LangBranchID
		
	/* Move properties from worktable to published table */
	INSERT INTO tblContentProperty 
		(fkPropertyDefinitionID,
		fkContentID,
		fkLanguageBranchID,
		ScopeName,
        ListIndex,
        BranchSpecificScope,
		[guid],
		Boolean,
		Number,
		FloatNumber,
		ContentType,
		ContentLink,
		Date,
		String,
		LongString,
		LongStringLength,
        LinkGuid)
	SELECT
		fkPropertyDefinitionID,
		@ContentID,
		@LangBranchID,
		ScopeName,
        ListIndex,
        BranchSpecificScope,
		[guid],
		Boolean,
		Number,
		FloatNumber,
		ContentType,
		ContentLink,
		Date,
		String,
		LongString,
		/* LongString is utf-16 - Datalength gives bytes, i e div by 2 gives characters */
		/* Include length to handle delayed loading of LongString with threshold */
		COALESCE(DATALENGTH(LongString), 0) / 2,
        LinkGuid
	FROM
		tblWorkContentProperty
	WHERE
		fkWorkContentID=@WorkContentID
	
	/* Move categories to published tables */
	DELETE 	tblContentCategory
	FROM tblContentCategory
	LEFT JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentCategory.CategoryType 
	WHERE 	tblContentCategory.fkContentID=@ContentID
			AND (NOT fkContentTypeID IS NULL OR CategoryType=0)
			AND (tblPropertyDefinition.LanguageSpecific>2 OR @IsMasterLang=1)--Only lang specific on non-master
			AND tblContentCategory.fkLanguageBranchID=@LangBranchID
			
	INSERT INTO tblContentCategory
		(fkContentID,
		fkCategoryID,
		CategoryType,
		fkLanguageBranchID,
		ScopeName)
	SELECT
		@ContentID,
		fkCategoryID,
		CategoryType,
		@LangBranchID,
		ScopeName
	FROM
		tblWorkContentCategory
	WHERE
		fkWorkContentID=@WorkContentID
	
	IF @ResetCommonDraft = 1
		EXEC editSetCommonDraftVersion @WorkContentID = @WorkContentID, @Force = 1				

	IF (@TrimVersions = 1)
        DELETE FROM tblWorkContent WHERE fkContentID = @ContentID AND fkLanguageBranchID = @LangBranchID AND Status = 5


    /*If the matser language we are updating*/
    IF @IsMasterLang = 1
    BEGIN
        /* Find other content language versions that is not master and is not published*/
        IF EXISTS(SELECT * FROM tblWorkContent WHERE fkContentID = @ContentID AND Status <> 4)
        BEGIN
            UPDATE
                tblWorkContent
            SET
                ArchiveContentGUID    = tblContent.ArchiveContentGUID,
                ChildOrderRule        = tblContent.ChildOrderRule,
                PeerOrder            =  tblContent.PeerOrder,
                VisibleInMenu        =  tblContent.VisibleInMenu
            FROM tblWorkContent
            INNER JOIN tblContent ON tblContent.pkID = @ContentID AND tblContent.fkMasterLanguageBranchID = @LangBranchID
            WHERE
                tblWorkContent.fkContentID = @ContentID AND tblWorkContent.fkLanguageBranchID != @LangBranchID AND tblWorkContent.Status <> 4 AND tblWorkContent.Status <> 5
        END
    END
END
GO


PRINT N'Altering Procedure [dbo].[netVersionFilterList]...';
GO

ALTER PROCEDURE [dbo].[netVersionFilterList]
(
	@StartIndex INT,
	@MaxRows INT,
	@ContentID INT = NULL,
	@ChangedBy NVARCHAR(255) = NULL,
	@ExcludeDeleted BIT = 0,
	@LanguageIds dbo.IDTable READONLY,
	@Statuses dbo.IDTable READONLY,
	@SortColumn varchar(40) = 'Saved',
	@SortDirection bit = 1,
	@Name NVARCHAR(255) = NULL,
    @StartDelayPublishUntil DATETIME = NULL,
    @EndDelayPublishUntil DATETIME = NULL
)
AS

BEGIN	
	SET NOCOUNT ON

    DECLARE @OrderBy NVARCHAR(64)

	DECLARE @StatusCount INT
	SELECT @StatusCount = COUNT(*) FROM @Statuses

	DECLARE @LanguageCount INT
	SELECT @LanguageCount = COUNT(*) FROM @LanguageIds;
	--Optimized for contentid with and without language, those are most used
	IF (@ContentID IS NOT NULL AND @ChangedBy IS NULL AND @ExcludeDeleted = 0 AND @StatusCount = 0 AND @Name IS NULL AND @StartDelayPublishUntil IS NULL AND @EndDelayPublishUntil IS NULL)
	BEGIN
        IF (@LanguageCount = 0)
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID
			)
            SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS 'TotalRows'
			FROM  TempResult
            UNION
			SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS 'TotalRows'
			FROM  TempResult
			WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
		END
		ELSE
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID AND
				    W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)

		)
        SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS 'TotalRows'
		FROM  TempResult
        UNION
		SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS 'TotalRows'
		FROM  TempResult
		WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
	 END
    END
	ELSE
	BEGIN
        SET @OrderBy =
		        CASE @SortColumn
			        WHEN 'Saved' THEN 'W.Saved'
			        WHEN 'Name' THEN 'W.Name'
			        WHEN 'ID' THEN 'W.fkContentID'
                    WHEN 'DelayPublishUntil' THEN 'W.DelayPublishUntil'
                    WHEN 'SavedBy' THEN 'W.ChangedByName'
                    WHEN 'StatusChangedBy' THEN 'W.NewStatusByName'
                    
		        END
        IF(@SortDirection = 1)
		        SET @OrderBy = @OrderBy + ' DESC'
        DECLARE @SqlQuery NVARCHAR(MAX) = '
		    WITH TempResult as
		    (
			    SELECT ROW_NUMBER() OVER(ORDER BY '+ @OrderBy +' ) as RowNumber,
				    W.fkContentID AS ContentID,
				    W.pkID AS WorkID,
				    W.Status AS VersionStatus,
				    W.ChangedByName AS SavedBy,
				    W.Saved AS ItemCreated,
				    W.Name,
				    W.fkLanguageBranchID as LanguageBranchID,
				    W.CommonDraft,
				    W.fkMasterVersionID as MasterVersion,
				    CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
				    W.NewStatusByName As StatusChangedBy,
				    W.DelayPublishUntil
			    FROM
				    tblWorkContent AS W
				    INNER JOIN
				    tblContent AS C ON C.pkID=W.fkContentID ' + CHAR(13);

	    DECLARE @IncludeAnd BIT = 0
	    IF @ContentID IS NOT NULL
	    BEGIN
		    SET @SqlQuery += 'WHERE W.fkContentID=@ContentID' + CHAR(13);
		    SET @IncludeAnd = 1
	    END

	    IF @ChangedBy IS NOT NULL
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.ChangedByName=@ChangedBy' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.ChangedByName=@ChangedBy' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @Name IS NOT NULL
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.Name LIKE @Name' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.Name LIKE @Name' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @StatusCount > 0
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @LanguageCount > 0
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @ExcludeDeleted = 1
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND C.Deleted = 0' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE C.Deleted = 0' + CHAR(13);
	    END

	    IF @StartDelayPublishUntil IS NOT NULL 
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
	    END

        IF @EndDelayPublishUntil IS NOT NULL 
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
	    END

	    SET @SqlQuery += ')' + CHAR(13);

	    SET @SqlQuery += 'SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS ''TotalRows''
		    FROM    TempResult
            UNION
            SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS ''TotalRows''
		    FROM    TempResult
		    WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)'
   		
	    print @SqlQuery
	    EXEC sp_executesql @SqlQuery, N'@StartIndex INT, @MaxRows INT, @ContentID INT, @ChangedBy NVARCHAR(255), @Name NVARCHAR(255), @Statuses dbo.IDTable READONLY, @LanguageIds dbo.IDTable READONLY, @ExcludeDeleted BIT, @StartDelayPublishUntil DATETIME, @EndDelayPublishUntil DATETIME', 
		    @StartIndex = @StartIndex, @MaxRows = @MaxRows, @ContentID = @ContentID, @ChangedBy = @ChangedBy, @Name = @Name, @Statuses = @Statuses, @LanguageIds = @LanguageIds, @ExcludeDeleted = @ExcludeDeleted, @StartDelayPublishUntil = @StartDelayPublishUntil, @EndDelayPublishUntil = @EndDelayPublishUntil
    END
END
GO

ALTER PROCEDURE dbo.netContentListPaged
(
	@Ids IDTable READONLY,
	@Threshold INT = 0,
	@LanguageBranchID INT
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @ContentItems ContentLanguageTable
    INSERT INTO @ContentItems(ContentID) SELECT ID FROM @Ids

	/* We need to know which languages exist */
	UPDATE @ContentItems SET 
		LanguageID = CASE WHEN fkLanguageBranchID IS NULL THEN fkMasterLanguageBranchID ELSE fkLanguageBranchID END
	FROM @ContentItems AS P
	INNER JOIN tblContent ON tblContent.pkID = P.ContentID
	LEFT JOIN tblContentLanguage ON P.ContentID = tblContentLanguage.fkContentID AND tblContentLanguage.fkLanguageBranchID = @LanguageBranchID

	/* Get all languages for all items*/
	SELECT tblContentLanguage.fkContentID as PageLinkID, tblContent.fkContentTypeID as PageTypeID, tblContentLanguage.fkLanguageBranchID as PageLanguageBranchID 
	FROM tblContentLanguage
	INNER JOIN @ContentItems on ContentID=tblContentLanguage.fkContentID
	INNER JOIN tblContent ON tblContent.pkID = tblContentLanguage.fkContentID
	ORDER BY tblContentLanguage.fkContentID

	/* Get all language versions that is requested (including master) */
	SELECT
		L.Status AS PageWorkStatus,
		L.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		CASE AutomaticLink
			WHEN 1 THEN
				(CASE
					WHEN L.ContentLinkGUID IS NULL THEN 0	/* EPnLinkNormal */
					WHEN L.FetchData=1 THEN 4				/* EPnLinkFetchdata */
					ELSE 1								/* EPnLinkShortcut */
				END)
			ELSE
				(CASE
					WHEN L.LinkURL=N'#' THEN 3				/* EPnLinkInactive */
					ELSE 2								/* EPnLinkExternal */
				END)
		END AS PageShortcutType,
		L.ExternalURL AS PageExternalURL,
		L.ContentLinkGUID AS PageShortcutLinkID,
		L.Name AS PageName,
		L.URLSegment AS PageURLSegment,
		L.LinkURL AS PageLinkURL,
		L.BlobUri,
		L.ThumbnailUri,
		L.Created AS PageCreated,
		L.Changed AS PageChanged,
		L.Saved AS PageSaved,
		L.StartPublish AS PageStartPublish,
		L.StopPublish AS PageStopPublish,
		CASE WHEN L.Status = 4 THEN CAST(0 AS BIT) ELSE CAST(1 AS BIT) END AS PagePendingPublish,
		L.CreatorName AS PageCreatedBy,
		L.ChangedByName AS PageChangedBy,
		L.fkFrameID AS PageTargetFrame,
		0 AS PageChangedOnPublish,
		0 AS PageDelayedPublish,
		L.fkLanguageBranchID AS PageLanguageBranchID,
		L.DelayPublishUntil AS PageDelayPublishUntil
	FROM @ContentItems AS P
	INNER JOIN tblContentLanguage AS L ON ContentID=L.fkContentID
	WHERE L.fkLanguageBranchID = P.LanguageID
	ORDER BY L.fkContentID

	IF (@@ROWCOUNT = 0)
	BEGIN
		RETURN
	END
		

/* Get data for page */
	SELECT
		ContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		fkParentID  AS PageParentLinkID,
		fkContentTypeID AS PageTypeID,
		NULL AS PageTypeName,
		CONVERT(INT,VisibleInMenu) AS PageVisibleInMenu,
		ChildOrderRule AS PageChildOrderRule,
		0 AS PagePeerOrderRule,	-- No longer used
		PeerOrder AS PagePeerOrder,
		CONVERT(NVARCHAR(38),tblContent.ContentGUID) AS PageGUID,
		ArchiveContentGUID AS PageArchiveLinkID,
		ContentAssetsID,
		ContentOwnerID,
		CONVERT(INT,Deleted) AS PageDeleted,
		DeletedBy AS PageDeletedBy,
		DeletedDate AS PageDeletedDate,
		fkMasterLanguageBranchID AS PageMasterLanguageBranchID,
		CreatorName
	FROM @ContentItems
	INNER JOIN tblContent ON ContentID=tblContent.pkID
	ORDER BY tblContent.pkID

	IF (@@ROWCOUNT = 0)
	BEGIN
		RETURN
	END;

    WITH DelayedScopes (ContentID , Scope)
    AS
    (SELECT ContentID, ScopeName from GetListPropertiesOverThreshold(@ContentItems, @Threshold))

	/* Get the properties */
	/* NOTE! The CASE:s for LongString and Guid uses the precomputed LongStringLength to avoid 
	referencing LongString which may slow down the query */
	SELECT
		tblContentProperty.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblPropertyDefinition.Name AS PropertyName,
		tblPropertyDefinition.pkID as PropertyDefinitionID,
		ScopeName,
		CONVERT(INT, Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		tblContentProperty.ContentType AS PageType,
		ContentLink,
		LinkGuid,	
		Date AS DateValue,
		String,
		(CASE 
			WHEN (@Threshold = 0) OR (COALESCE(LongStringLength, 2147483647) < @Threshold) THEN
				LongString
			ELSE
				NULL
		END) AS LongString,
		tblContentProperty.fkLanguageBranchID AS PageLanguageBranchID,
		(CASE 
			WHEN (@Threshold = 0) OR (COALESCE(LongStringLength, 2147483647) < @Threshold) THEN
				NULL
			ELSE
				guid
		END) AS Guid,
        (CASE 
			WHEN (ScopeName IS NOT NULL AND ListIndex IS NOT NULL AND ds.Scope IS NOT NULL) THEN
				ds.Scope
			ELSE
				NULL
		END) AS DelayedScope
	FROM @ContentItems AS P
	INNER JOIN tblContent ON tblContent.pkID=P.ContentID
	INNER JOIN tblContentProperty WITH (NOLOCK) ON tblContent.pkID=tblContentProperty.fkContentID --The join with tblContent ensures data integrity
	INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentProperty.fkPropertyDefinitionID
    LEFT JOIN DelayedScopes ds ON (P.ContentID = ds.ContentID AND tblContentProperty.ScopeName LIKE ds.Scope + '%')
	WHERE NOT tblPropertyDefinition.fkContentTypeID IS NULL AND
		(tblContentProperty.fkLanguageBranchID = P.LanguageID
	OR
		((tblContentProperty.BranchSpecificScope IS NULL AND tblPropertyDefinition.LanguageSpecific < 3) OR tblContentProperty.BranchSpecificScope = 0))
    AND (ScopeName is NULL OR ListIndex IS NULL OR ListIndex = 0 OR ds.Scope IS NULL)
	ORDER BY tblContent.pkID;

	/*Get category information*/
	SELECT 
		fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		fkCategoryID,
		CategoryType
	FROM tblContentCategory
	INNER JOIN @ContentItems ON ContentID=tblContentCategory.fkContentID
	WHERE CategoryType=0
	ORDER BY fkContentID,fkCategoryID

	/* Get access information */
	SELECT
		fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblContentAccess.Name,
		IsRole,
		AccessMask
	FROM
		@ContentItems
	INNER JOIN 
	    tblContentAccess ON ContentID=tblContentAccess.fkContentID
	ORDER BY
		fkContentID
END
GO


ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8013

GO

PRINT N'Update complete.';
GO
