--beginvalidatingquery
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
begin
	declare @ver int
	exec @ver=sp_DatabaseVersion
	if (@ver >= 8014)
		select 0, 'Already correct database version'
	else if (@ver = 8013)
		select 1, 'Upgrading database'
	else
		select -1, 'Invalid database version detected'
end
else
	select -1, 'Not an EPiServer database'
--endvalidatingquery

GO
PRINT N'Creating User-Defined Table Type [dbo].[InlineBlockUsageTable]...';


GO
CREATE TYPE [dbo].[InlineBlockUsageTable] AS TABLE (
    [ContentTypeID]              INT            NOT NULL,
    [ScopeName]                  NVARCHAR (450) NOT NULL,
    [PropertyDefinitionID]       INT            NOT NULL,
    [ParentPropertyDefinitionID] INT            NULL);

GO
PRINT N'Creating Table [dbo].[tblInlineBlockUsage]...';


GO
CREATE TABLE [dbo].[tblInlineBlockUsage] (
    [pkID] [bigint] IDENTITY (1, 1) NOT NULL ,
    [fkContentTypeID] [int] NOT NULL ,
    [fkContentID] [int] NOT NULL ,
    [fkWorkContentID] [int] NOT NULL ,
    [ScopeName] NVARCHAR(450) NOT NULL, 
    [fkPropertyDefinitionID] INT NOT NULL,
    [fkParentPropertyDefinitionID] INT NULL, 
    CONSTRAINT [PK_tblInlineBlockUsage] PRIMARY KEY NONCLUSTERED 
    (
	    [pkID]
    ),
    CONSTRAINT [FK_tblInlineBlockUsage_tblContentType] FOREIGN KEY 
    (
	    [fkContentTypeID]
    ) 
    REFERENCES [tblContentType]
    (
	    [pkID]
    ),

    CONSTRAINT [FK_tblInlineBlockUsage_tblContent] FOREIGN KEY 
    (
	    [fkContentID]
    ) 
    REFERENCES [tblContent]
    (
	    [pkID]
    )
    ON DELETE CASCADE,

    CONSTRAINT [FK_tblInlineBlockUsage_tblWorkContent] FOREIGN KEY 
    (
	    [fkWorkContentID]
    ) 
    REFERENCES [tblWorkContent]
    (
	    [pkID]
    )
    ON DELETE CASCADE
)
GO

CREATE CLUSTERED INDEX [IDX_tblInlineBlockUsage_fkContentTypeID] ON [dbo].[tblInlineBlockUsage] 
(
	[fkContentTypeID] ASC
)
GO

CREATE NONCLUSTERED INDEX [IDX_tblInlineBlockUsage_ScopeName] ON [dbo].[tblInlineBlockUsage] 
(
	[ScopeName] ASC
)
GO

CREATE NONCLUSTERED INDEX [IDX_tblInlineBlockUsage_fkPropertyDefinitionID] ON [dbo].[tblInlineBlockUsage] 
(
	[fkPropertyDefinitionID] ASC
)
GO

PRINT N'Creating Function [dbo].[GetScopesForInlineBlocks]...';
GO

CREATE FUNCTION [dbo].[GetScopesForInlineBlocks] 
(
	@PropertyDefinitionID int
)
RETURNS @ScopedPropertiesTable TABLE 
(
	ScopeName nvarchar(450)
)
AS
BEGIN
    WITH ScopeNames(ScopeName, fkPropertyDefinitionID, Level) AS (
      SELECT ScopeName,
      fkPropertyDefinitionID,
      0 AS Level
      FROM tblInlineBlockUsage
      WHERE fkPropertyDefinitionID = @PropertyDefinitionID
      UNION ALL
      SELECT subEntries.ScopeName, subEntries.fkPropertyDefinitionID, s.Level + 1 as Level
      FROM tblInlineBlockUsage subEntries INNER JOIN
      ScopeNames s ON subEntries.fkParentPropertyDefinitionID = s.fkPropertyDefinitionID
      WHERE s.Level <= 20
      -- There is a theoretic possibility to nest areas in a way that is cyclic therefore we break at 20 levels, in reality should there never be such deep "real" inlineblock hierarchies 
     )
    INSERT INTO @ScopedPropertiesTable
    SELECT DISTINCT ScopeName FROM ScopeNames

    RETURN
END
GO

PRINT N'Altering Table [dbo].[tblProject]. Adding Allias ...';
GO

ALTER TABLE [dbo].[tblProject]
    ADD [Alias]  NVARCHAR (255)  NULL,
        [Source] NVARCHAR (255) NULL,
        [Saved]  DATETIME       NULL;
GO

PRINT N'Updating Table [dbo].[tblProject]...';
GO

UPDATE tblProject SET Saved = Created, Alias = REPLACE(NEWID(), '-', '') WHERE Alias IS  NULL
GO


PRINT N'Altering Table [dbo].[tblProject] alter Alias column as Unique...';
GO

ALTER TABLE [dbo].[tblProject]	ALTER COLUMN Alias  NVARCHAR (255) NOT NULL
ALTER TABLE [dbo].[tblProject]	ALTER COLUMN [Source] NVARCHAR (255) NULL
ALTER TABLE [dbo].[tblProject]	ALTER COLUMN [Saved]  DATETIME       NOT NULL;
ALTER TABLE [dbo].[tblProject]	ADD CONSTRAINT UQ_PRODUCT_ALIAS UNIQUE(Alias)
GO



PRINT N'Altering Procedure [dbo].[netProjectGet]...';
GO
ALTER PROCEDURE [dbo].[netProjectGet]
	@ID INT,
    @Alias NVARCHAR(255) = NULL
AS
BEGIN

    Declare @Result Table (pkID INT, IsPublic BIT, [Name] NVARCHAR(255), Created DATETIME, CreatedBy NVARCHAR(255), [Status] INT, DelayPublishUntil DATETIME, PublishingTrackingToken NVARCHAR(255), Alias NVARCHAR(255), [Source] NVARCHAR(255), Saved DATETIME)

    IF @Alias IS NOT NULL
    BEGIN
        INSERT INTO @Result
	        SELECT pkID, IsPublic, Name, Created, CreatedBy, [Status], DelayPublishUntil, PublishingTrackingToken, Alias, [Source], Saved
	        FROM tblProject
	        WHERE tblProject.Alias = @Alias
    END
    ELSE
    BEGIN
        INSERT INTO @Result   
	        SELECT pkID, IsPublic, Name, Created, CreatedBy, [Status], DelayPublishUntil, PublishingTrackingToken, Alias, [Source], Saved
	        FROM tblProject
	        WHERE tblProject.pkID = @ID
    END

     SELECT pkID, IsPublic, Name, Created, CreatedBy, [Status], DelayPublishUntil, PublishingTrackingToken, Alias, [Source], Saved FROM @Result

    SELECT tblProjectMember.pkID, tblProjectMember.Name, Type
	FROM tblProjectMember
    INNER JOIN @Result ON tblProjectMember.fkProjectID = [@Result].[pkID]
	ORDER BY tblProjectMember.Name ASC
END

GO

PRINT N'Altering Procedure [dbo].[netProjectList]...';
GO

ALTER PROCEDURE [dbo].[netProjectList]
	@StartIndex INT = 0,
	@MaxRows INT,
	@Status INT = -1
AS
BEGIN
	SET NOCOUNT ON;
	DECLARE @projectIDs TABLE(projectID INT NOT NULL, TotalRows INT NOT NULL);

	WITH PageCTE AS
    (SELECT pkID, [Status],
     ROW_NUMBER() OVER(ORDER BY Name ASC, pkID ASC) AS intRow
     FROM tblProject
	 WHERE @Status  = -1 OR @Status = [Status])

	INSERT INTO  @projectIDs 
		SELECT PageCTE.pkID, (SELECT COUNT(*) FROM PageCTE) AS 'TotalRows' 
		FROM PageCTE 
		WHERE intRow BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)

	--Projects
	SELECT 
		pkID, Name, IsPublic, CreatedBy, Created, [Status], DelayPublishUntil, PublishingTrackingToken, Alias, [Source], Saved
	FROM 
		tblProject 
	INNER JOIN @projectIDs AS projects ON projects.projectID = tblProject.pkID


	--ProjectMembers
	SELECT 
		pkID, projectID, Name, Type
	FROM 
		tblProjectMember 
	INNER JOIN @projectIDs AS projects ON projects.projectID = tblProjectMember.fkProjectID
	ORDER BY projectID, Name

	RETURN COALESCE((SELECT TOP 1 TotalRows FROM @projectIDs), 0)

END
GO

PRINT N'Altering Procedure [dbo].[netProjectSave]...';
GO

ALTER PROCEDURE [dbo].[netProjectSave]
	@ID INT,
	@Name	nvarchar(255),
	@IsPublic BIT,
	@Created	datetime,
	@CreatedBy	nvarchar(255),
	@Status INT,
	@DelayPublishUntil datetime = NULL,
	@PublishingTrackingToken nvarchar(255),
	@Members dbo.ProjectMemberTable READONLY,
    @Alias nvarchar(255),
    @Source nvarchar(255) = NULL,
    @Saved	datetime
AS
BEGIN
	SET NOCOUNT ON

	IF @ID=0
	BEGIN
		INSERT INTO tblProject(Name, IsPublic, Created, CreatedBy, [Status], DelayPublishUntil, PublishingTrackingToken, Alias, [Source], Saved) VALUES(@Name, @IsPublic, @Created, @CreatedBy, @Status, @DelayPublishUntil, @PublishingTrackingToken, @Alias, @Source, @Saved)
		SET @ID = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE tblProject SET Name=@Name, IsPublic=@IsPublic, [Status] = @Status, DelayPublishUntil = @DelayPublishUntil, PublishingTrackingToken = @PublishingTrackingToken, [Source] = @Source, Saved = @Saved  WHERE pkID=@ID
	END

	MERGE tblProjectMember AS Target
    USING @Members AS Source
    ON (Target.pkID = Source.ID AND Target.fkProjectID=@ID)
    WHEN MATCHED THEN 
        UPDATE SET Name = Source.Name, Type = Source.Type
	WHEN NOT MATCHED BY Source AND Target.fkProjectID = @ID THEN
		DELETE
	WHEN NOT MATCHED BY Target THEN
		INSERT (fkProjectID, Name, Type)
		VALUES (@ID, Source.Name, Source.Type);


	SELECT pkID, Name, Type
	FROM tblProjectMember
	WHERE tblProjectMember.fkProjectID = @ID
	ORDER BY Name ASC

	RETURN @ID
END
GO

PRINT N'Refreshing Procedure [dbo].[netProjectDelete]...';
GO

EXECUTE sp_refreshsqlmodule N'[dbo].[netProjectDelete]';
GO


PRINT N'Altering Table [dbo].[tblPropertyDefinition]...';
GO


ALTER TABLE [dbo].[tblPropertyDefinition] 
	ADD Saved DATETIME NULL 
GO


PRINT N'Updating Table [dbo].[tblPropertyDefinition]...';
GO

UPDATE [dbo].tblPropertyDefinition SET Saved = GETUTCDATE() WHERE Saved IS  NULL
GO

PRINT N'Altering Table [dbo].[tblPropertyDefinition] alter saved not null...';
GO

ALTER TABLE [dbo].[tblPropertyDefinition]	ALTER COLUMN Saved DATETIME NOT NULL
GO

PRINT N'Altering view [dbo].[tblPageDefinition]...';
GO

ALTER VIEW [dbo].[tblPageDefinition]
AS
SELECT  [pkID],
		[fkContentTypeID] AS fkPageTypeID,
		[fkPropertyDefinitionTypeID] AS fkPageDefinitionTypeID,
		[FieldOrder],
		[Name],
		[Property],
        [IsList],
		[Required],
		[Advanced],
		[Searchable],
		[EditCaption],
		[HelpText],
		[ObjectProgID],
		[DefaultValueType],
		[LongStringSettings],
		[SettingsID],
		[LanguageSpecific],
		[DisplayEditUI],
		[ExistsOnModel],
        [EditorHint],
        [ImageDescriptor],
        [Saved]
FROM    dbo.tblPropertyDefinition
GO


PRINT N'Altering Procedure [dbo].[netPageDefinitionGet]...';
GO

ALTER PROCEDURE [dbo].[netPageDefinitionGet]
(
	@PageDefinitionID INT
)
AS
BEGIN
	SELECT tblPageDefinition.pkID AS ID,
		fkPageTypeID AS PageTypeID,
		COALESCE(fkPageDefinitionTypeID,tblPageDefinition.Property) AS PageDefinitionTypeID,
		tblPageDefinition.Name,
		COALESCE(tblPageDefinitionType.Property,tblPageDefinition.Property) AS Type,
        IsList,
		CONVERT(INT,Required) AS Required,
		Advanced,
		CONVERT(INT,Searchable) AS Searchable,
		DefaultValueType,
		EditCaption,
		HelpText,
		ObjectProgID,
		LongStringSettings,
		SettingsID,
		CONVERT(INT,Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		PageType,
		PageLink,
		Date AS DateValue,
		String,
		LongString,
		FieldOrder,
		LanguageSpecific,
		DisplayEditUI,
		ExistsOnModel,
        EditorHint,
        ImageDescriptor,
        Saved
	FROM tblPageDefinition
	LEFT JOIN tblPropertyDefault ON tblPropertyDefault.fkPageDefinitionID=tblPageDefinition.pkID
	LEFT JOIN tblPageDefinitionType ON tblPageDefinitionType.pkID=tblPageDefinition.fkPageDefinitionTypeID
	WHERE tblPageDefinition.pkID = @PageDefinitionID
END
GO


PRINT N'Altering Procedure [dbo].[netPageDefinitionList]...';
GO

ALTER PROCEDURE [dbo].[netPageDefinitionList]
(
	@PageTypeID INT = NULL
)
AS
BEGIN
	SELECT tblPageDefinition.pkID AS ID,
		fkPageTypeID AS PageTypeID,
		COALESCE(fkPageDefinitionTypeID,tblPageDefinition.Property) AS PageDefinitionTypeID,
		tblPageDefinition.Name,
		COALESCE(tblPageDefinitionType.Property,tblPageDefinition.Property) AS Type,
		CONVERT(INT,Required) AS Required,
        IsList,
		Advanced,
		CONVERT(INT,Searchable) AS Searchable,
		DefaultValueType,
		EditCaption,
		HelpText,
		ObjectProgID,
		LongStringSettings,
		SettingsID,
		CONVERT(INT,Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		PageType,
		PageLink,
		Date AS DateValue,
		String,
		LongString,
		NULL AS OldType,
		FieldOrder,
		LanguageSpecific,
		DisplayEditUI,
		ExistsOnModel,
        EditorHint,
        ImageDescriptor,
        Saved
	FROM tblPageDefinition
	LEFT JOIN tblPropertyDefault ON tblPropertyDefault.fkPageDefinitionID=tblPageDefinition.pkID
	LEFT JOIN tblPageDefinitionType ON tblPageDefinitionType.pkID=tblPageDefinition.fkPageDefinitionTypeID
	WHERE (fkPageTypeID = @PageTypeID) OR (fkPageTypeID IS NULL AND @PageTypeID IS NULL)
	ORDER BY FieldOrder,tblPageDefinition.pkID
END
GO

PRINT N'Altering Procedure [dbo].[netPageDefinitionSave]...';
GO

ALTER PROCEDURE dbo.netPageDefinitionSave
(
	@PageDefinitionID      INT OUTPUT,
	@PageTypeID            INT,
	@Name                  NVARCHAR(100),
	@PageDefinitionTypeID  INT,
	@Required              BIT = NULL,
    @IsList                BIT = NULL,
	@Advanced              INT = NULL,
	@Searchable            BIT = NULL,
	@DefaultValueType      INT = NULL,
	@EditCaption           NVARCHAR(255) = NULL,
	@HelpText              NVARCHAR(2000) = NULL,
	@ObjectProgID          NVARCHAR(255) = NULL,
	@LongStringSettings    INT = NULL,
	@SettingsID            UNIQUEIDENTIFIER = NULL,
	@FieldOrder            INT = NULL,
	@Type                  INT = NULL OUTPUT,
	@OldType               INT = NULL OUTPUT,
	@LanguageSpecific      INT = 0,
	@DisplayEditUI         BIT = NULL,
	@ExistsOnModel         BIT = 0,
    @EditorHint            NVARCHAR(255) = NULL,
    @ImageDescriptor       NVARCHAR(50) = NULL,
    @Saved                 DATETIME
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	SELECT @OldType = tblPropertyDefinitionType.Property 
	FROM tblPropertyDefinition
	INNER JOIN tblPropertyDefinitionType ON tblPropertyDefinitionType.pkID=tblPropertyDefinition.fkPropertyDefinitionTypeID
	WHERE tblPropertyDefinition.pkID=@PageDefinitionID

	SELECT @Type = Property FROM tblPropertyDefinitionType WHERE pkID=@PageDefinitionTypeID
	IF @Type IS NULL
		RAISERROR('Cannot find data type',16,1)
	IF @PageTypeID=0
		SET @PageTypeID = NULL

	IF @PageDefinitionID = 0 AND @ExistsOnModel = 1
	BEGIN
        --We want to block parallel inserts which can happen if several applications initializes in parallel, hence we read all table with update lock
        DECLARE @NotUsed INT
        SELECT @NotUsed = pkID FROM tblPropertyDefinition WITH (UPDLOCK, HOLDLOCK)
		
        SET @PageDefinitionID = ISNULL((SELECT pkID FROM tblPropertyDefinition where Name = @Name AND fkContentTypeID = @PageTypeID), @PageDefinitionID)
	END

	IF @PageDefinitionID=0
	BEGIN	
		INSERT INTO tblPropertyDefinition
		(
			fkContentTypeID,
			fkPropertyDefinitionTypeID,
			Name,
			Property,
			Required,
            IsList,
			Advanced,
			Searchable,
			DefaultValueType,
			EditCaption,
			HelpText,
			ObjectProgID,
			LongStringSettings,
			SettingsID,
			FieldOrder,
			LanguageSpecific,
			DisplayEditUI,
			ExistsOnModel,
            EditorHint,
            ImageDescriptor,
            Saved
		)
		VALUES
		(
			@PageTypeID,
			@PageDefinitionTypeID,
			@Name,
			@Type,
			@Required,
            @IsList,
			@Advanced,
			@Searchable,
			@DefaultValueType,
			@EditCaption,
			@HelpText,
			@ObjectProgID,
			@LongStringSettings,
			@SettingsID,
			@FieldOrder,
			@LanguageSpecific,
			@DisplayEditUI,
			@ExistsOnModel,
            @EditorHint,
            @ImageDescriptor,
            @Saved
		)
		SET @PageDefinitionID =  SCOPE_IDENTITY() 
	END
	ELSE
	BEGIN
		UPDATE tblPropertyDefinition SET
			Name 		= @Name,
			fkPropertyDefinitionTypeID	= @PageDefinitionTypeID,
			Property 	= @Type,
			Required 	= @Required,
            IsList      = @IsList,
			Advanced 	= @Advanced,
			Searchable 	= @Searchable,
			DefaultValueType = @DefaultValueType,
			EditCaption 	= @EditCaption,
			HelpText 	= @HelpText,
			ObjectProgID 	= @ObjectProgID,
			LongStringSettings = @LongStringSettings,
			SettingsID = @SettingsID,
			LanguageSpecific = @LanguageSpecific,
			FieldOrder = @FieldOrder,
			DisplayEditUI = @DisplayEditUI,
			ExistsOnModel = @ExistsOnModel,
            EditorHint = @EditorHint,
            ImageDescriptor = @ImageDescriptor,
            Saved = @Saved
		WHERE pkID=@PageDefinitionID

        DELETE FROM tblPropertyDefault WHERE fkPageDefinitionID=@PageDefinitionID
	    IF @LanguageSpecific<3
	    BEGIN
		    /* NOTE: Here we take into consideration that language neutral dynamic properties are always stored on language 
			    with id 1 (which perhaps should be changed and in that case the special handling here could be removed). */
		    IF @PageTypeID IS NULL
		    BEGIN
			    DELETE tblProperty
			    FROM tblProperty
			    INNER JOIN tblPage ON tblPage.pkID=tblProperty.fkPageID
			    WHERE fkPageDefinitionID=@PageDefinitionID AND tblProperty.fkLanguageBranchID<>1
		    END
		    ELSE
		    BEGIN
			    DELETE tblProperty
			    FROM tblProperty
			    INNER JOIN tblPage ON tblPage.pkID=tblProperty.fkPageID
			    WHERE fkPageDefinitionID=@PageDefinitionID AND tblProperty.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID
		    END
		    DELETE tblWorkProperty
		    FROM tblWorkProperty
		    INNER JOIN tblWorkPage ON tblWorkProperty.fkWorkPageID=tblWorkPage.pkID
		    INNER JOIN tblPage ON tblPage.pkID=tblWorkPage.fkPageID
		    WHERE fkPageDefinitionID=@PageDefinitionID AND tblWorkPage.fkLanguageBranchID<>tblPage.fkMasterLanguageBranchID

		    DELETE 
			    tblCategoryPage
		    FROM
			    tblCategoryPage
		    INNER JOIN
			    tblPage
		    ON
			    tblPage.pkID = tblCategoryPage.fkPageID
		    WHERE
			    CategoryType = @PageDefinitionID
		    AND
			    tblCategoryPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID

		    DELETE 
			    tblWorkCategory
		    FROM
			    tblWorkCategory
		    INNER JOIN 
			    tblWorkPage
		    ON
			    tblWorkCategory.fkWorkPageID = tblWorkPage.pkID
		    INNER JOIN
			    tblPage
		    ON
			    tblPage.pkID = tblWorkPage.fkPageID
		    WHERE
			    CategoryType = @PageDefinitionID
		    AND
			    tblWorkPage.fkLanguageBranchID <> tblPage.fkMasterLanguageBranchID
	    END
	END
END
GO

PRINT N'Altering Procedure [dbo].[netContentTypeDelete]...';


GO
ALTER PROCEDURE dbo.netContentTypeDelete
(
	@ContentTypeID		INT
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	
	/* Do not try to delete a type that is in use */
    DECLARE @ContentID INT;
    DECLARE @ContentInUse TABLE(ContentID INT, WorkID INT)

    /* check for instances of content type */
    INSERT INTO @ContentInUse SELECT pkID as ContentID, 0 as WorkID
	FROM tblContent
	INNER JOIN tblContentLanguage ON tblContentLanguage.fkContentID=tblContent.pkID
	WHERE fkContentTypeID=@ContentTypeID AND tblContentLanguage.fkLanguageBranchID=tblContent.fkMasterLanguageBranchID

    SELECT TOP 1 @ContentID = ContentID FROM @ContentInUse
    IF (@ContentID IS NULL)
        /* check if contenttype is used as property */
        INSERT INTO @ContentInUse EXEC dbo.netBlockTypeCheckUsage @BlockTypeID = @ContentTypeID

    SELECT TOP 1 ContentID FROM @ContentInUse
	IF (@@ROWCOUNT <> 0)
        RETURN 1

	/* If the content type is used in a content definition, it can't be deleted */
	DECLARE @ContentTypeGuid UNIQUEIDENTIFIER
	SET @ContentTypeGuid = (SELECT ContentType.ContentTypeGUID
	FROM tblContentType as ContentType 
	WHERE ContentType.pkID=@ContentTypeID)
	
	DECLARE @PropertyDefinitionTypeID INT
	SET @PropertyDefinitionTypeID = (SELECT pkID FROM tblPropertyDefinitionType WHERE fkContentTypeGUID = @ContentTypeGuid)
	
	SELECT ContentType.pkID AS ContentTypeID, ContentType.Name AS ContentTypeName 
	FROM tblContentType AS ContentType
	INNER JOIN tblPropertyDefinition AS PropertyDefinition ON ContentType.pkID = PropertyDefinition.fkContentTypeID
	WHERE PropertyDefinition.fkPropertyDefinitionTypeID = @PropertyDefinitionTypeID
	IF (@@ROWCOUNT <> 0)
		RETURN 1
	
	DELETE FROM 
		tblContentTypeDefault
	WHERE 
		fkContentTypeID=@ContentTypeID

	DELETE FROM 
		tblWorkContentProperty
	FROM 
		tblWorkContentProperty AS WP
	INNER JOIN 
		tblPropertyDefinition AS PD ON WP.fkPropertyDefinitionID=PD.pkID 
	WHERE 
		PD.Property=3 AND 
		ContentType=@ContentTypeID

	DELETE FROM 
		tblContentProperty
	FROM 
		tblContentProperty AS P
	INNER JOIN 
		tblPropertyDefinition AS PD ON P.fkPropertyDefinitionID=PD.pkID 
	WHERE 
		PD.Property=3 AND 
		ContentType=@ContentTypeID

	DELETE FROM 
		tblContentTypeToContentType 
	WHERE 
		fkContentTypeParentID=@ContentTypeID OR 
		fkContentTypeChildID=@ContentTypeID
		
	DELETE FROM 
		tblPropertyDefinition 
	WHERE 
		fkContentTypeID=@ContentTypeID

	DELETE FROM 
		tblPropertyDefinitionType
	FROM 
		tblPropertyDefinitionType
	INNER JOIN tblContentType ON tblPropertyDefinitionType.fkContentTypeGUID = tblContentType.ContentTypeGUID
	WHERE
		tblContentType.pkID=@ContentTypeID

    DELETE FROM 
		tblInlineBlockUsage
	WHERE
		fkContentTypeID = @ContentTypeID

	DELETE FROM 
		tblContentType
	WHERE
		pkID=@ContentTypeID
	
	RETURN 0
END
GO

PRINT N'Altering Procedure [dbo].[netBlockTypeGetUsage]...';


GO
ALTER PROCEDURE [dbo].[netBlockTypeGetUsage]
(
	@BlockTypeID		INT,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	SET NOCOUNT ON

	IF (@OnlyPublished = 1)
	BEGIN
        -- This captures when block has properties that are stored directly on block
         SELECT DISTINCT
			tblContentProperty.fkContentID AS ContentID, 
			0 AS WorkID,
            tblContentLanguage.Name,
			tblLanguageBranch.LanguageID AS LanguageBranch
		FROM 
			tblContentProperty 
		INNER JOIN
            tblPropertyDefinition ON tblContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
            tblContentLanguage ON tblContentProperty.fkContentID = tblContentLanguage.fkContentID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblContentProperty.fkLanguageBranchID
        WHERE
            tblPropertyDefinition.fkContentTypeID = @BlockTypeID
        UNION
        -- This captures block properties (list or single)
		SELECT DISTINCT
			Property.fkContentID as ContentID, 
			0 AS WorkID,
			ContentLanguage.Name,
			LanguageBranch.LanguageID AS LanguageBranch
		FROM tblContentProperty as Property WITH(INDEX(IDX_tblContentProperty_ScopeName))
		INNER JOIN
            dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		INNER JOIN
            tblContentLanguage as ContentLanguage ON Property.fkContentID=ContentLanguage.fkContentID
		INNER JOIN
            tblLanguageBranch as LanguageBranch ON LanguageBranch.pkID=ContentLanguage.fkLanguageBranchID
        UNION
        -- This capture usage of inline blocks.
        SELECT DISTINCT
            tblInlineBlockUsage.fkContentID as ContentID, 
            0 as WorkID,
            tblWorkContent.Name,
            tblLanguageBranch.LanguageID AS LanguageBranch
        FROM tblInlineBlockUsage
        INNER JOIN
            tblWorkContent ON tblInlineBlockUsage.fkWorkContentID = tblWorkContent.pkID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblWorkContent.fkLanguageBranchID
        WHERE tblInlineBlockUsage.fkContentTypeID = @BlockTypeID
        
	END
	ELSE
	BEGIN
         -- This captures when block has properties that are stored directly on block
        SELECT DISTINCT
			tblWorkContent.fkContentID AS ContentID, 
			tblWorkContent.pkID AS WorkID,
            tblWorkContent.Name,
			tblLanguageBranch.LanguageID AS LanguageBranch
		FROM 
			tblWorkContentProperty 
		INNER JOIN
            tblPropertyDefinition ON tblWorkContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
            tblWorkContent ON tblWorkContentProperty.fkWorkContentID = tblWorkContent.pkID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblWorkContent.fkLanguageBranchID
        WHERE
            tblPropertyDefinition.fkContentTypeID = @BlockTypeID
        UNION
        -- This capture the rare case when a block has no values stored on it self but has nested block (list or single)
		SELECT DISTINCT
			WorkContent.fkContentID as ContentID, 
			WorkContent.pkID as WorkID,
			WorkContent.Name,
			LanguageBranch.LanguageID AS LanguageBranch
		FROM tblWorkContentProperty as Property WITH(INDEX(IDX_tblWorkContentProperty_ScopeName))
		INNER JOIN
            dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		INNER JOIN
            tblWorkContent as WorkContent ON WorkContent.pkID = Property.fkWorkContentID
		INNER JOIN
            tblLanguageBranch as LanguageBranch ON LanguageBranch.pkID=WorkContent.fkLanguageBranchID
        UNION
        -- This capture the case when an inline block has no properties stored on it self but has nested block (list or single)
        SELECT DISTINCT
            tblInlineBlockUsage.fkContentID as ContentID, 
            tblInlineBlockUsage.fkWorkContentID as WorkID,
            tblWorkContent.Name,
            tblLanguageBranch.LanguageID AS LanguageBranch
        FROM tblInlineBlockUsage
        INNER JOIN
            tblWorkContent ON tblInlineBlockUsage.fkWorkContentID = tblWorkContent.pkID
        INNER JOIN
            tblLanguageBranch ON tblLanguageBranch.pkID = tblWorkContent.fkLanguageBranchID
        WHERE tblInlineBlockUsage.fkContentTypeID = @BlockTypeID
	END
END
GO


PRINT N'Altering Procedure [dbo].[netBlockTypeCheckUsage]...';


GO

ALTER PROCEDURE [dbo].[netBlockTypeCheckUsage]
(
	@BlockTypeID		INT,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE @ContentID INT;
    DECLARE @WorkID INT;
    DECLARE @InlineBlockScopes TABLE(ScopeName nvarchar(450));

	IF (@OnlyPublished = 1)
	BEGIN
         -- This captures when block has properties that are stored directly on block
        SELECT TOP 1
			@ContentID = tblContentProperty.fkContentID, 
			@WorkID = 0
		FROM 
			tblContentProperty 
		INNER JOIN 
			tblPropertyDefinition ON tblContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        WHERE tblPropertyDefinition.fkContentTypeID = @BlockTypeID

         IF (@ContentID IS NOT NULL)
            BEGIN
                SELECT
                    @ContentID AS ContentID,
                    @WorkID AS WorkID
            END
            ELSE
            BEGIN
                -- This capture inline blocks used in content area items
                SELECT TOP 1
	                @ContentID = fkContentID, 
	                @WorkID = fkWorkContentID
                FROM tblInlineBlockUsage
                WHERE fkContentTypeID = @BlockTypeID
		    END
            IF (@ContentID IS NOT NULL)
            BEGIN
                SELECT
                    @ContentID AS ContentID,
                    @WorkID AS WorkID
            END
            ELSE
            BEGIN
                -- This capture block properties (single or lists)
                SELECT TOP 1
			        Property.fkContentID as ContentID, 
			        0 as WorkID
		        FROM tblContentProperty as Property WITH(INDEX(IDX_tblContentProperty_ScopeName))
		        INNER JOIN dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON 
			        Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
            END
	END
	ELSE
	BEGIN
        -- This captures when block has properties that are stored directly on block
        SELECT TOP 1
			@ContentID = tblWorkContent.fkContentID, 
			@WorkID = tblWorkContent.pkID
		FROM 
			tblWorkContentProperty 
		INNER JOIN 
			tblPropertyDefinition ON tblWorkContentProperty.fkPropertyDefinitionID = tblPropertyDefinition.pkID
        INNER JOIN
			tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
        WHERE tblPropertyDefinition.fkContentTypeID = @BlockTypeID

        IF (@ContentID IS NOT NULL)
        BEGIN
            SELECT
                @ContentID AS ContentID,
                @WorkID AS WorkID
        END
        ELSE
        BEGIN
            -- This capture inline blocks used in content area items
            SELECT TOP 1
	            @ContentID = fkContentID, 
	            @WorkID = fkWorkContentID
            FROM tblInlineBlockUsage
            WHERE fkContentTypeID = @BlockTypeID

            IF (@ContentID IS NOT NULL)
            BEGIN
                SELECT
                    @ContentID AS ContentID,
                    @WorkID AS WorkID
            END
            ELSE
            BEGIN
                -- This capture block properties (single or lists)
                SELECT TOP 1
			        WorkContent.fkContentID as ContentID, 
			        WorkContent.pkID as WorkID
		        FROM tblWorkContentProperty as Property WITH(INDEX(IDX_tblWorkContentProperty_ScopeName))
		        INNER JOIN dbo.GetScopedBlockProperties(@BlockTypeID) as ScopedProperties ON 
			        Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
		        INNER JOIN
			        tblWorkContent as WorkContent ON Property.fkWorkContentID=WorkContent.pkID
            END
        END
	END
END
GO

PRINT N'Altering Procedure [dbo].[netConvertPropertyForPageType]...';


GO

ALTER PROCEDURE [dbo].[netConvertPropertyForPageType]
(
	@PageID		INT,
	@FromPageType	INT,
	@FromPropertyID 	INT,
	@ToPropertyID		INT,
	@Recursive		BIT,
	@MasterLanguageID INT,
	@IsTest			BIT
)
AS
BEGIN

	DECLARE @cnt INT;
	DECLARE @LanguageSpecific INT
	DECLARE @LanguageSpecificSource INT
	DECLARE @IsBlock BIT
    DECLARE @IsList BIT
    DECLARE @IsContentAreaOrItem BIT
	SET @LanguageSpecific = 0
	SET @LanguageSpecificSource = 0
	SET @IsBlock = 0
    SET @IsList = 0
    SET @IsContentAreaOrItem = 0

	CREATE TABLE  #updatepages(fkChildID int)
 
	INSERT INTO #updatepages(fkChildID)  
	SELECT fkChildID 
	FROM tblTree tree
	INNER JOIN tblPage page
	ON page.pkID = tree.fkChildID 
	WHERE @Recursive = 1
	AND tree.fkParentID = @PageID
	AND page.fkPageTypeID = @FromPageType
	UNION (SELECT pkID FROM tblPage WHERE pkID = @PageID AND fkPageTypeID = @FromPageType)

	IF @IsTest = 1
	BEGIN	
		SET @cnt = (	SELECT COUNT(*)
				FROM tblProperty 
				WHERE (fkPageDefinitionID = @FromPropertyID
				or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID))
			+ (	SELECT COUNT(*)
				FROM tblWorkProperty 
				WHERE (fkPageDefinitionID = @FromPropertyID
				or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
				AND EXISTS (
				SELECT * 
					FROM tblWorkPage 
					WHERE pkID = fkWorkPageID
					AND  EXISTS (
						SELECT * from #updatepages WHERE fkChildID=fkPageID)
				))
		IF @ToPropertyID IS NULL OR @ToPropertyID = 0-- mark deleted rows with -
			SET @cnt = -@cnt
	END
	ELSE
	BEGIN
		IF @ToPropertyID IS NULL OR @ToPropertyID = 0-- no definition exists for the new page type for this property so remove it
		BEGIN
			DELETE
			FROM tblProperty 
			WHERE (fkPageDefinitionID = @FromPropertyID
			or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
			AND  EXISTS (
				SELECT * from #updatepages WHERE fkChildID=fkPageID)

			SET @cnt = -@@rowcount

			DELETE 
			FROM tblWorkProperty 
			WHERE (fkPageDefinitionID = @FromPropertyID
			or ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '[.(:]%')
			AND EXISTS (
				SELECT * 
				FROM tblWorkPage 
				WHERE pkID = fkWorkPageID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)
				)
			SET @cnt = @cnt-@@rowcount

            DECLARE @ScopedProperties TABLE (ScopeName nvarchar(450))
	        INSERT INTO @ScopedProperties SELECT * FROM dbo.GetExistingScopesForDefinition(@FromPropertyID)
            DELETE FROM tblInlineBlockUsage WHERE ScopeName IN (SELECT ScopeName FROM @ScopedProperties)
            AND EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
            SET @cnt = @cnt-@@rowcount
		END 	
		ELSE IF @FromPropertyID IS NOT NULL -- from property exists and has to be replaced
		BEGIN
			-- Need to check if the property we're converting to is unique for each language or not
			SELECT @LanguageSpecific = LanguageSpecific 
			FROM tblPageDefinition 
			WHERE pkID = @ToPropertyID

			-- Need to check if the property we're converting from is unique for each language or not
			SELECT @LanguageSpecificSource = LanguageSpecific 
			FROM tblPageDefinition 
			WHERE pkID = @FromPropertyID
			
			-- Need to check if the property we're converting is a block (Property 12 is a block)
			SELECT @IsBlock = CAST(count(*) as bit)
			FROM tblPageDefinition 
			Where pkID = @FromPropertyID and Property = 12

            -- Need to check if the property we're converting is a list
			SELECT @IsList = CAST(count(*) as bit)
			FROM tblPageDefinition 
			Where pkID = @FromPropertyID and IsList = 1

             -- Need to check if the property we're converting is a ContentArea or ContentAreaItem
			SELECT @IsContentAreaOrItem = CAST(count(*) as bit)
			FROM tblPageDefinition INNER JOIN tblPageDefinitionType on tblPageDefinition.fkPageDefinitionTypeID = tblPageDefinitionType.pkID
			WHERE tblPageDefinition.pkID = @FromPropertyID AND
            tblPageDefinitionType.TypeName LIKE 'EPiServer.SpecializedProperties.PropertyContentArea%'

            DECLARE @DefinitionTypeFrom int
		    DECLARE @DefinitionTypeTo int

            IF @IsList = 1
			BEGIN
				SET @DefinitionTypeFrom = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@FromPropertyID)
				SET @DefinitionTypeTo = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@ToPropertyID)
				IF (@DefinitionTypeFrom <> @DefinitionTypeTo)
				BEGIN
					RAISERROR (N'Property definitions are not of same list type.', 16, 1)
					RETURN 0
				END
				
				-- Update older versions of block
				-- update scopename in tblWorkProperty
                 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, '(')
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '(%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 SET @cnt = @@rowcount

				-- Update current version of block
				-- update scopename in tblProperty
				
                 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, '(')
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '(%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				 SET @cnt = @cnt + @@rowcount
			END

            IF @IsContentAreaOrItem = 1
			BEGIN

				-- Update older versions of contentarea
				-- update scopename in tblWorkProperty
                 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, ':')
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + ':%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 SET @cnt = @@rowcount

				-- Update current version of contentarea
				-- update scopename in tblProperty
				
                 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, ':')
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + ':%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				 SET @cnt = @cnt + @@rowcount

                 UPDATE tblInlineBlockUsage
                 SET ScopeName = dbo.ConvertIndexedScopeName(ScopeName,@FromPropertyID, @ToPropertyID, ':')
                 WHERE  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
                 SET @cnt = @cnt + @@rowcount

                 UPDATE tblInlineBlockUsage
                 SET fkPropertyDefinitionID = @ToPropertyID
                 WHERE fkPropertyDefinitionID = @FromPropertyID
                 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
                 SET @cnt = @cnt + @@rowcount

                 UPDATE tblInlineBlockUsage
                 SET fkParentPropertyDefinitionID = @ToPropertyID
                 WHERE fkParentPropertyDefinitionID = @FromPropertyID
                 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
                 SET @cnt = @cnt + @@rowcount
			END

			IF @IsBlock = 1
			BEGIN
				SET @DefinitionTypeFrom = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@FromPropertyID)
				SET @DefinitionTypeTo = 
					(SELECT fkPageDefinitionTypeID FROM tblPageDefinition WHERE pkID =@ToPropertyID)
				IF (@DefinitionTypeFrom <> @DefinitionTypeTo)
				BEGIN
					RAISERROR (N'Property definitions are not of same block type.', 16, 1)
					RETURN 0
				END
				
				-- Update older versions of block
				-- update scopename in tblWorkProperty
				
				 UPDATE tblWorkProperty 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 FROM tblWorkProperty prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
			
				 SET @cnt = @@rowcount

				-- Update current version of block
				-- update scopename in tblProperty
				
				 UPDATE tblProperty 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt + @@rowcount

                 --Update category tables in case block has category properties
                 UPDATE tblWorkContentCategory
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 FROM tblWorkContentCategory prop
				 INNER JOIN tblWorkPage wpa ON prop.fkWorkContentID = wpa.pkID
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)

                 UPDATE tblContentCategory 
				 SET ScopeName = dbo.ConvertScopeName(ScopeName,@FromPropertyID, @ToPropertyID)
				 WHERE ScopeName LIKE '%.' + CAST(@FromPropertyID as varchar) + '.%'
				 AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkContentID)
			END
			ELSE -- Not a block
			BEGIN
				-- Update older versions
				UPDATE tblWorkProperty SET fkPageDefinitionID = @ToPropertyID
					FROM tblWorkProperty prop
					INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
					WHERE prop.fkPageDefinitionID = @FromPropertyID
					AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
			
				SET @cnt = @@rowcount

				-- Update current version 
				UPDATE tblProperty SET fkPageDefinitionID = @ToPropertyID
				WHERE fkPageDefinitionID = @FromPropertyID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt + @@rowcount
			END

			IF (@LanguageSpecific < 3 AND @LanguageSpecificSource > 2)
			BEGIN
				-- The destination property is not language specific which means
				-- that we need to remove all of the old properties in other
				-- languages that could not be mapped
				DELETE FROM tblWorkProperty
					FROM tblWorkProperty prop
					INNER JOIN tblWorkPage wpa ON prop.fkWorkPageID = wpa.pkID
					WHERE (prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
					or prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%')
					AND wpa.fkLanguageBranchID <> @MasterLanguageID
					AND EXISTS (SELECT * from #updatepages WHERE fkChildID=wpa.fkPageID)
				
				SET @cnt = @cnt - @@rowcount		
				
				DELETE FROM tblProperty 
				WHERE (fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				or ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%')
				AND fkLanguageBranchID <> @MasterLanguageID
				AND  EXISTS (
					SELECT * from #updatepages WHERE fkChildID=fkPageID)

				SET @cnt = @cnt - @@rowcount				
			END
			ELSE IF (@LanguageSpecificSource < 3)
			BEGIN
				-- Converting from language neutral to language supporting property
				-- We must copy existing master language property to other languages for the page
				
				-- NOTE: Due to the way language neutral properties are loaded, that is they are always
				-- loaded from published version, see netPageDataLoadVersion it is sufficient to add property
				-- values to tblProperty (no need to update tblWorkProperty
				
				INSERT INTO tblProperty
					(fkPageDefinitionID,
					fkPageID,
					fkLanguageBranchID,
					ScopeName,
                    ListIndex,
                    BranchSpecificScope,
					Boolean,
					Number,
					FloatNumber,
					PageType,
					PageLink,
					LinkGuid,
					Date,
					String,
					LongString,
					LongStringLength)
				SELECT 
					CASE @IsBlock when 1 then Prop.fkPageDefinitionID else @ToPropertyID end, 
					Prop.fkPageID,
					Lang.fkLanguageBranchID,
					Prop.ScopeName,
                    Prop.ListIndex,
                    Prop.BranchSpecificScope,
					Prop.Boolean,
					Prop.Number,
					Prop.FloatNumber,
					Prop.PageType,
					Prop.PageLink,
					Prop.LinkGuid,
					Prop.Date,
					Prop.String,
					Prop.LongString,
					Prop.LongStringLength
				FROM
				tblPageLanguage Lang
				INNER JOIN
				tblProperty Prop ON Prop.fkLanguageBranchID = @MasterLanguageID
				WHERE
				Prop.fkPageID = @PageID AND
				(Prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				or Prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%') AND
				Prop.fkLanguageBranchID = @MasterLanguageID AND
				Lang.fkLanguageBranchID <> @MasterLanguageID AND
				Lang.fkPageID = @PageID

				-- Need to add entries to tblWorkProperty for all pages not in the master language
				-- First we need to read the master language property into a temp table
				CREATE TABLE #TempWorkProperty
				(
					fkPageDefinitionID int,
					ScopeName nvarchar(450),
                    ListIndex int,
                    BranchSpecificScope bit,
					Boolean bit,
					Number int,
					FloatNumber float,
					PageType int,
					PageLink int,
				    LinkGuid uniqueidentifier,
					Date datetime,
					String nvarchar(450),
					LongString nvarchar(max)
				)

				INSERT INTO #TempWorkProperty
				SELECT
					Prop.fkPageDefinitionID,
					Prop.ScopeName,
                    Prop.ListIndex,
                    Prop.BranchSpecificScope,
					Prop.Boolean,
					Prop.Number,
					Prop.FloatNumber,
					Prop.PageType,
					Prop.PageLink,
				    Prop.LinkGuid,
					Prop.Date,
					Prop.String,
					Prop.LongString
				FROM
					tblWorkProperty AS Prop
					INNER JOIN
					tblWorkPage AS Page ON Prop.fkWorkPageID = Page.pkID
				WHERE
					(Prop.fkPageDefinitionID = @ToPropertyID -- already converted to new type!
				    or Prop.ScopeName LIKE '%.' + CAST(@ToPropertyID as varchar) + '[.(:]%') AND
					Page.fkLanguageBranchID = @MasterLanguageID AND
					Page.fkPageID = @PageID
					ORDER BY Page.pkID DESC

				-- Now add a new property for every language (except master) using the master value
				INSERT INTO	tblWorkProperty 
				SELECT
					CASE @IsBlock when 1 then TempProp.fkPageDefinitionID else @ToPropertyID end,
					Page.pkID,
					TempProp.ScopeName,
                    TempProp.ListIndex,
                    TempProp.BranchSpecificScope,
					TempProp.Boolean,
					TempProp.Number,
					TempProp.FloatNumber,
					TempProp.PageType,
					TempProp.PageLink,
					TempProp.Date,
					TempProp.String,
					TempProp.LongString,
					TempProp.LinkGuid
				FROM 
					tblWorkPage AS Page
					CROSS JOIN #TempWorkProperty AS TempProp
				WHERE
                    Page.fkPageID = @PageID AND
					Page.fkLanguageBranchID <> @MasterLanguageID

				DROP TABLE #TempWorkProperty

			END
		END
	END

	DROP TABLE #updatepages

	RETURN (@cnt)
END
GO

PRINT N'Altering Procedure [dbo].[netContentLoadScopedProperties]...';
GO

ALTER PROCEDURE dbo.netContentLoadScopedProperties
(
    @ContentID INT,
    @ScopeName NVARCHAR(450),
    @LanguageBranchID INT = NULL,
    @PropertyDefinitionID INT = NULL
)
AS
BEGIN
	SELECT
		tblContentProperty.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblPropertyDefinition.Name AS PropertyName,
		tblPropertyDefinition.pkID as PropertyDefinitionID,
		ScopeName,
		CONVERT(INT, Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		tblContentProperty.ContentType AS PageType,
		ContentLink,
		LinkGuid,	
		Date AS DateValue,
		String,
		LongString,
		tblContentProperty.fkLanguageBranchID AS PageLanguageBranchID,
		NULL AS Guid,
        0 AS DelayedList
	FROM tblContentProperty
	INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentProperty.fkPropertyDefinitionID
	WHERE tblContentProperty.fkContentID = @ContentID
    AND tblContentProperty.fkPropertyDefinitionID = @PropertyDefinitionID
    AND NOT tblPropertyDefinition.fkContentTypeID IS NULL AND
		(@LanguageBranchID IS NULL
    OR
        tblContentProperty.fkLanguageBranchID = @LanguageBranchID
	OR
		tblContentProperty.BranchSpecificScope = 0)
    UNION
    SELECT
		tblContentProperty.fkContentID AS PageLinkID,
		NULL AS PageLinkWorkID,
		tblPropertyDefinition.Name AS PropertyName,
		tblPropertyDefinition.pkID as PropertyDefinitionID,
		ScopeName,
		CONVERT(INT, Boolean) AS Boolean,
		Number AS IntNumber,
		FloatNumber,
		tblContentProperty.ContentType AS PageType,
		ContentLink,
		LinkGuid,	
		Date AS DateValue,
		String,
		LongString,
		tblContentProperty.fkLanguageBranchID AS PageLanguageBranchID,
		NULL AS Guid,
        0 AS DelayedList
	FROM tblContentProperty
	INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblContentProperty.fkPropertyDefinitionID
	WHERE tblContentProperty.fkContentID = @ContentID
    AND ScopeName LIKE @ScopeName + '%'
    AND NOT tblPropertyDefinition.fkContentTypeID IS NULL AND
		(tblContentProperty.fkLanguageBranchID = @LanguageBranchID
	OR
		tblContentProperty.BranchSpecificScope = 0)
END
GO

PRINT N'Altering Procedure [dbo].[netVersionFilterList]...';
GO
ALTER PROCEDURE [dbo].[netVersionFilterList]
(
	@StartIndex INT,
	@MaxRows INT,
	@ContentID INT = NULL,
	@ChangedBy NVARCHAR(255) = NULL,
	@ExcludeDeleted BIT = 0,
	@LanguageIds dbo.IDTable READONLY,
	@Statuses dbo.IDTable READONLY,
	@SortColumn varchar(40) = 'Saved',
	@SortDirection bit = 1,
	@Name NVARCHAR(255) = NULL,
    @StartDelayPublishUntil DATETIME = NULL,
    @EndDelayPublishUntil DATETIME = NULL
)
AS

BEGIN	
	SET NOCOUNT ON

    DECLARE @OrderBy NVARCHAR(64)
    DECLARE @UnionOrderBy NVARCHAR(64)

	DECLARE @StatusCount INT
	SELECT @StatusCount = COUNT(*) FROM @Statuses

	DECLARE @LanguageCount INT
	SELECT @LanguageCount = COUNT(*) FROM @LanguageIds;
	--Optimized for contentid with and without language, those are most used
	IF (@ContentID IS NOT NULL AND @ChangedBy IS NULL AND @ExcludeDeleted = 0 AND @StatusCount = 0 AND @Name IS NULL AND @StartDelayPublishUntil IS NULL AND @EndDelayPublishUntil IS NULL)
	BEGIN
        IF (@LanguageCount = 0)
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID
			)
            SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS 'TotalRows'
			FROM  TempResult
            UNION
			SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS 'TotalRows'
			FROM  TempResult
			WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
            ORDER BY ItemCreated DESC
		END
		ELSE
		BEGIN
			WITH TempResult as
			(
				SELECT ROW_NUMBER() OVER(ORDER BY W.Saved DESC) as RowNumber,
					W.fkContentID AS ContentID,
					W.pkID AS WorkID,
					W.Status AS VersionStatus,
					W.ChangedByName AS SavedBy,
					W.Saved AS ItemCreated,
					W.Name,
					W.fkLanguageBranchID as LanguageBranchID,
					W.CommonDraft,
					W.fkMasterVersionID as MasterVersion,
					CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
					W.NewStatusByName As StatusChangedBy,
					W.DelayPublishUntil
				FROM
					tblWorkContent AS W
					INNER JOIN
					tblContent AS C ON C.pkID=W.fkContentID
				WHERE
					W.fkContentID=@ContentID AND
				    W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)

		    )
            SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS 'TotalRows'
		    FROM  TempResult
            UNION
		    SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS 'TotalRows'
		    FROM  TempResult
		    WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
            ORDER BY ItemCreated DESC
	    END
    END
	ELSE
	BEGIN
        SET @OrderBy =
		        CASE @SortColumn
			        WHEN 'Saved' THEN 'W.Saved'
			        WHEN 'Name' THEN 'W.Name'
			        WHEN 'ID' THEN 'W.fkContentID'
                    WHEN 'DelayPublishUntil' THEN 'W.DelayPublishUntil'
                    WHEN 'SavedBy' THEN 'W.ChangedByName'
                    WHEN 'StatusChangedBy' THEN 'W.NewStatusByName'
                    
		        END
        -- The ORDER BY is used in the final UNION statement. Need to order by TotalRows first to make sure the "total row" always on top of result set
        SET @UnionOrderBy =
		        CASE @SortColumn
			        WHEN 'Saved' THEN 'ItemCreated'
			        WHEN 'Name' THEN 'Name'
			        WHEN 'ID' THEN 'ContentID'
                    WHEN 'DelayPublishUntil' THEN 'DelayPublishUntil'
                    WHEN 'SavedBy' THEN 'SavedBy'
                    WHEN 'StatusChangedBy' THEN 'StatusChangedBy'
                    
		        END
        IF(@SortDirection = 1)
        BEGIN
		    SET @OrderBy = @OrderBy + ' DESC'
            SET @UnionOrderBy = @UnionOrderBy + ' DESC'
        END

        DECLARE @SqlQuery NVARCHAR(MAX) = '
		    WITH TempResult as
		    (
			    SELECT ROW_NUMBER() OVER(ORDER BY '+ @OrderBy +' ) as RowNumber,
				    W.fkContentID AS ContentID,
				    W.pkID AS WorkID,
				    W.Status AS VersionStatus,
				    W.ChangedByName AS SavedBy,
				    W.Saved AS ItemCreated,
				    W.Name,
				    W.fkLanguageBranchID as LanguageBranchID,
				    W.CommonDraft,
				    W.fkMasterVersionID as MasterVersion,
				    CASE WHEN C.fkMasterLanguageBranchID=W.fkLanguageBranchID THEN 1 ELSE 0 END AS IsMasterLanguageBranch,
				    W.NewStatusByName As StatusChangedBy,
				    W.DelayPublishUntil
			    FROM
				    tblWorkContent AS W
				    INNER JOIN
				    tblContent AS C ON C.pkID=W.fkContentID ' + CHAR(13);

	    DECLARE @IncludeAnd BIT = 0
	    IF @ContentID IS NOT NULL
	    BEGIN
		    SET @SqlQuery += 'WHERE W.fkContentID=@ContentID' + CHAR(13);
		    SET @IncludeAnd = 1
	    END

	    IF @ChangedBy IS NOT NULL
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.ChangedByName=@ChangedBy' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.ChangedByName=@ChangedBy' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @Name IS NOT NULL
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.Name LIKE @Name' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.Name LIKE @Name' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @StatusCount > 0
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.Status IN (SELECT ID FROM @Statuses)' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @LanguageCount > 0
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.fkLanguageBranchID IN (SELECT ID FROM @LanguageIds)' + CHAR(13);
		    SET @IncludeAnd = 1
	    END
	    IF @ExcludeDeleted = 1
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND C.Deleted = 0' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE C.Deleted = 0' + CHAR(13);
	    END

	    IF @StartDelayPublishUntil IS NOT NULL 
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.DelayPublishUntil >= @StartDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
	    END

        IF @EndDelayPublishUntil IS NOT NULL 
	    BEGIN
		    IF @IncludeAnd = 1
			    SET @SqlQuery += 'AND W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
		    ELSE
			    SET @SqlQuery += 'WHERE W.DelayPublishUntil <= @EndDelayPublishUntil' + CHAR(13);
            SET @IncludeAnd = 1
	    END

	    SET @SqlQuery += ')' + CHAR(13);

	    SET @SqlQuery += 'SELECT NULL AS ContentID, NULL AS WorkID, NULL AS VersionStatus, NULL AS SavedBy, NULL AS ItemCreated, NULL AS Name, NULL AS LanguageBranchID, NULL AS CommonDraft, NULL AS MasterVersion, NULL AS IsMasterLanguageBranch, NULL AS StatusChangedBy, NULL AS DelayPublishUntil, COUNT(*)  AS ''TotalRows''
		    FROM    TempResult
            UNION
            SELECT  ContentID, WorkID, VersionStatus, SavedBy, ItemCreated, Name, LanguageBranchID, CommonDraft, MasterVersion, IsMasterLanguageBranch, StatusChangedBy, DelayPublishUntil, NULL AS ''TotalRows''
		    FROM    TempResult
		    WHERE RowNumber BETWEEN (@StartIndex +1) AND (@MaxRows + @StartIndex)
            ORDER BY ' + @UnionOrderBy

   		
	    print @SqlQuery
	    EXEC sp_executesql @SqlQuery, N'@StartIndex INT, @MaxRows INT, @ContentID INT, @ChangedBy NVARCHAR(255), @Name NVARCHAR(255), @Statuses dbo.IDTable READONLY, @LanguageIds dbo.IDTable READONLY, @ExcludeDeleted BIT, @StartDelayPublishUntil DATETIME, @EndDelayPublishUntil DATETIME', 
		    @StartIndex = @StartIndex, @MaxRows = @MaxRows, @ContentID = @ContentID, @ChangedBy = @ChangedBy, @Name = @Name, @Statuses = @Statuses, @LanguageIds = @LanguageIds, @ExcludeDeleted = @ExcludeDeleted, @StartDelayPublishUntil = @StartDelayPublishUntil, @EndDelayPublishUntil = @EndDelayPublishUntil
    END
END
GO

PRINT N'Altering Procedure [dbo].[editCreateContentVersion]...';


GO
ALTER PROCEDURE [dbo].[editCreateContentVersion]
(
	@ContentID			INT,
	@WorkContentID		INT,
	@UserName		NVARCHAR(255),
	@MaxVersions	INT = NULL,
	@SavedDate		DATETIME,
	@LanguageBranch	NCHAR(17)
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	
	DECLARE @NewWorkContentID		INT
	DECLARE @DeleteWorkContentID	INT
	DECLARE @ObsoleteVersions	INT
	DECLARE @retval				INT
	DECLARE @IsMasterLang		BIT
	DECLARE @LangBranchID		INT
	
	SELECT @LangBranchID = pkID FROM tblLanguageBranch WHERE LanguageID=@LanguageBranch
	IF @LangBranchID IS NULL
	BEGIN
		RAISERROR (N'editCreateContentVersion: LanguageBranchID is null, possibly empty table tblLanguageBranch', 16, 1, @WorkContentID)
		RETURN 0
	END

	IF (@WorkContentID IS NULL OR @WorkContentID=0 )
	BEGIN
		/* If we have a published version use it, else the latest saved version */
		IF EXISTS(SELECT * FROM tblContentLanguage WHERE Status = 4 AND fkContentID=@ContentID AND fkLanguageBranchID=@LangBranchID)
			SELECT @WorkContentID=[Version] FROM tblContentLanguage WHERE fkContentID=@ContentID AND fkLanguageBranchID=@LangBranchID
		ELSE
			SELECT TOP 1 @WorkContentID=pkID FROM tblWorkContent WHERE fkContentID=@ContentID AND fkLanguageBranchID=@LangBranchID ORDER BY Saved DESC
	END

	IF EXISTS( SELECT * FROM tblContent WHERE pkID=@ContentID AND fkMasterLanguageBranchID IS NULL )
		UPDATE tblContent SET fkMasterLanguageBranchID=@LangBranchID WHERE pkID=@ContentID
	
	SELECT @IsMasterLang = CASE WHEN @LangBranchID=fkMasterLanguageBranchID THEN 1 ELSE 0 END FROM tblContent WHERE pkID=@ContentID
		
		/* Create a new version of this content */
		INSERT INTO tblWorkContent
			(fkContentID,
			fkMasterVersionID,
			ChangedByName,
			ContentLinkGUID,
			fkFrameID,
			ArchiveContentGUID,
			Name,
			LinkURL,
			ExternalURL,
			VisibleInMenu,
			LinkType,
			Created,
			Saved,
			StartPublish,
			StopPublish,
			ChildOrderRule,
			PeerOrder,
			fkLanguageBranchID,
			URLSegment,
			ThumbnailUri,
			BlobUri)
		SELECT 
			fkContentID,
			@WorkContentID,
			@UserName,
			ContentLinkGUID,
			fkFrameID,
			ArchiveContentGUID,
			Name,
			LinkURL,
			ExternalURL,
			VisibleInMenu,
			LinkType,
			Created,
			@SavedDate,
			StartPublish,
			StopPublish,
			ChildOrderRule,
			PeerOrder,
			@LangBranchID,
			URLSegment,
			ThumbnailUri,
			BlobUri
		FROM 
			tblWorkContent 
		WHERE 
			pkID=@WorkContentID
	
		IF (@@ROWCOUNT = 1)
		BEGIN
			/* Remember version number */
			SET @NewWorkContentID= SCOPE_IDENTITY() 
			/* Copy all properties as well */
			INSERT INTO tblWorkContentProperty
				(fkPropertyDefinitionID,
				fkWorkContentID,
				ScopeName,
                ListIndex,
                BranchSpecificScope,
				Boolean,
				Number,
				FloatNumber,
				ContentType,
				ContentLink,
				Date,
				String,
				LongString,
                LinkGuid)          
			SELECT
				fkPropertyDefinitionID,
				@NewWorkContentID,
				ScopeName,
                ListIndex,
                BranchSpecificScope,
				Boolean,
				Number,
				FloatNumber,
				ContentType,
				ContentLink,
				Date,
				String,
				LongString,
                LinkGuid
			FROM
				tblWorkContentProperty
			INNER JOIN tblPropertyDefinition ON tblPropertyDefinition.pkID=tblWorkContentProperty.fkPropertyDefinitionID
			WHERE
				fkWorkContentID=@WorkContentID
				AND (tblPropertyDefinition.LanguageSpecific>2 OR @IsMasterLang=1)--Only lang specific on non-master

            INSERT INTO tblInlineBlockUsage
                (fkContentID,
                fkWorkContentID,
                fkContentTypeID,
                ScopeName,
                fkPropertyDefinitionID,
                fkParentPropertyDefinitionID)
            SELECT
                fkContentID,
                @NewWorkContentID,
                fkContentTypeID,
                ScopeName,
                fkPropertyDefinitionID,
                fkParentPropertyDefinitionID
            FROM tblInlineBlockUsage
            WHERE fkWorkContentID = @WorkContentID
				
			/* Finally take care of categories */
			INSERT INTO tblWorkContentCategory
				(fkWorkContentID,
				fkCategoryID,
				CategoryType,
				ScopeName)
			SELECT
				@NewWorkContentID,
				fkCategoryID,
				CategoryType,
				ScopeName
			FROM
				tblWorkContentCategory
			WHERE
				fkWorkContentID=@WorkContentID
				AND (CategoryType<>0 OR @IsMasterLang=1)--No content category on languages
		END
		ELSE
		BEGIN
			/* We did not have anything corresponding to the WorkContentID, create new work content from tblContent */
			INSERT INTO tblWorkContent
				(fkContentID,
				ChangedByName,
				ContentLinkGUID,
				fkFrameID,
				ArchiveContentGUID,
				Name,
				LinkURL,
				ExternalURL,
				VisibleInMenu,
				LinkType,
				Created,
				Saved,
				StartPublish,
				StopPublish,
				ChildOrderRule,
				PeerOrder,
				fkLanguageBranchID,
				URLSegment,
				ThumbnailUri,
				BlobUri)
			SELECT 
				@ContentID,
				COALESCE(@UserName, tblContentLanguage.CreatorName),
				tblContentLanguage.ContentLinkGUID,
				tblContentLanguage.fkFrameID,
				tblContent.ArchiveContentGUID,
				tblContentLanguage.Name,
				tblContentLanguage.LinkURL,
				tblContentLanguage.ExternalURL,
				tblContent.VisibleInMenu,
				CASE tblContentLanguage.AutomaticLink 
					WHEN 1 THEN 
						(CASE
							WHEN tblContentLanguage.ContentLinkGUID IS NULL THEN 0	/* EPnLinkNormal */
							WHEN tblContentLanguage.FetchData=1 THEN 4				/* EPnLinkFetchdata */
							ELSE 1								/* EPnLinkShortcut */
						END)
					ELSE
						(CASE 
							WHEN tblContentLanguage.LinkURL=N'#' THEN 3				/* EPnLinkInactive */
							ELSE 2								/* EPnLinkExternal */
						END)
				END AS LinkType ,
				tblContentLanguage.Created,
				@SavedDate,
				tblContentLanguage.StartPublish,
				tblContentLanguage.StopPublish,
				tblContent.ChildOrderRule,
				tblContent.PeerOrder,
				@LangBranchID,
				tblContentLanguage.URLSegment,
				ThumbnailUri,
				BlobUri
			FROM tblContentLanguage
			INNER JOIN tblContent ON tblContent.pkID=tblContentLanguage.fkContentID
			WHERE 
				tblContentLanguage.fkContentID=@ContentID AND tblContentLanguage.fkLanguageBranchID=@LangBranchID

			IF (@@ROWCOUNT = 1)
			BEGIN
				/* Remember version number */
				SET @NewWorkContentID= SCOPE_IDENTITY() 
				/* Copy all non-dynamic properties as well */
				INSERT INTO tblWorkContentProperty
					(fkPropertyDefinitionID,
					fkWorkContentID,
					ScopeName,
					Boolean,
					Number,
					FloatNumber,
					ContentType,
					ContentLink,
					Date,
					String,
					LongString,
                    LinkGuid)
				SELECT
					P.fkPropertyDefinitionID,
					@NewWorkContentID,
					P.ScopeName,
					P.Boolean,
					P.Number,
					P.FloatNumber,
					P.ContentType,
					P.ContentLink,
					P.Date,
					P.String,
					P.LongString,
                    P.LinkGuid
				FROM
					tblContentProperty AS P
				INNER JOIN
					tblPropertyDefinition AS PD ON P.fkPropertyDefinitionID=PD.pkID
				WHERE
					P.fkContentID=@ContentID AND (PD.fkContentTypeID IS NOT NULL)
					AND P.fkLanguageBranchID = @LangBranchID
					AND (PD.LanguageSpecific>2 OR @IsMasterLang=1)--Only lang specific on non-master

                INSERT INTO tblInlineBlockUsage
                    (fkContentID,
                    fkWorkContentID,
                    fkContentTypeID,
                    ScopeName,
                    fkPropertyDefinitionID,
                    fkParentPropertyDefinitionID)
                SELECT
                    BU.fkContentID,
                    @NewWorkContentID,
                    BU.fkContentTypeID,
                    BU.ScopeName,
                    BU.fkPropertyDefinitionID,
                    BU.fkParentPropertyDefinitionID
                FROM tblInlineBlockUsage AS BU
                INNER JOIN tblWorkContent ON BU.fkWorkContentID = tblWorkContent.pkID
                WHERE BU.fkContentID = @ContentID AND tblWorkContent.fkLanguageBranchID = @LangBranchID
					
				/* Finally take care of categories */
				INSERT INTO tblWorkContentCategory
					(fkWorkContentID,
					fkCategoryID,
					CategoryType)
				SELECT DISTINCT
					@NewWorkContentID,
					fkCategoryID,
					CategoryType
				FROM
					tblContentCategory
				LEFT JOIN
					tblPropertyDefinition AS PD ON tblContentCategory.CategoryType = PD.pkID
				WHERE
					tblContentCategory.fkContentID=@ContentID 
					AND (PD.fkContentTypeID IS NOT NULL OR tblContentCategory.CategoryType = 0) --Not dynamic properties
					AND (PD.LanguageSpecific=1 OR @IsMasterLang=1) --No content category on languages
			END
			ELSE
			BEGIN
				RAISERROR (N'Failed to create new version for content %d', 16, 1, @ContentID)
				RETURN 0
			END
		END

	/*If there is no version set for tblContentLanguage set it to this version*/
	UPDATE tblContentLanguage SET Version = @NewWorkContentID
	WHERE fkContentID = @ContentID AND fkLanguageBranchID = @LangBranchID AND Version IS NULL
		
	RETURN @NewWorkContentID
END
GO

PRINT N'Creating Procedure [dbo].[netInlineBlockUsageSave]...';


GO

CREATE PROCEDURE [dbo].[netInlineBlockUsageSave]
(
    @ContentID		    INT,
    @WorkContentID		INT,
    @InlineBlockUsage   dbo.InlineBlockUsageTable READONLY
)
AS
BEGIN
    SET NOCOUNT ON
    SET XACT_ABORT ON

    MERGE tblInlineBlockUsage AS Target
    USING @InlineBlockUsage AS SOURCE
    ON (Target.fkWorkContentID = @WorkContentID
        AND Target.fkContentID = @ContentID
        AND Target.fkContentTypeID = SOURCE.ContentTypeID
        AND Target.fkPropertyDefinitionID = SOURCE.PropertyDefinitionID
        AND Target.ScopeName = SOURCE.ScopeName)
    WHEN NOT MATCHED BY Target THEN
    INSERT (fkWorkContentID,
		    fkContentID,
            fkContentTypeID,
            ScopeName,
            fkPropertyDefinitionID,
            fkParentPropertyDefinitionID)
    VALUES (@WorkContentID,
		    @ContentID,
            SOURCE.ContentTypeID,
            SOURCE.ScopeName,
            SOURCE.PropertyDefinitionID,
            SOURCE.ParentPropertyDefinitionID)
    WHEN NOT MATCHED BY SOURCE
    AND Target.fkContentID = @ContentID AND Target.fkWorkContentID = @WorkContentID
    THEN DELETE;
END
GO


PRINT N'Altering Function [dbo].[GetExistingScopesForDefinition]...';


GO
ALTER FUNCTION [dbo].[GetExistingScopesForDefinition] 
(
	@PropertyDefinitionID int
)
RETURNS @ScopedPropertiesTable TABLE 
(
	ScopeName nvarchar(450)
)
AS
BEGIN
    INSERT INTO @ScopedPropertiesTable SELECT * FROM dbo.GetScopesForInlineBlocks(@PropertyDefinitionID)
    
	--Get blocktype if property is block property
	DECLARE @ContentTypeID INT;
	SET @ContentTypeID = (SELECT tblContentType.pkID FROM 
		tblPropertyDefinition
		INNER JOIN tblPropertyDefinitionType ON tblPropertyDefinition.fkPropertyDefinitionTypeID = tblPropertyDefinitionType.pkID
		INNER JOIN tblContentType ON tblPropertyDefinitionType.fkContentTypeGUID = tblContentType.ContentTypeGUID
		WHERE tblPropertyDefinition.pkID = @PropertyDefinitionID);
		
	IF (@ContentTypeID IS NOT NULL)
	BEGIN
		INSERT INTO @ScopedPropertiesTable
		SELECT DISTINCT Property.ScopeName FROM
			tblWorkContentProperty as Property WITH(INDEX(IDX_tblWorkContentProperty_ScopeName))
			INNER JOIN dbo.GetScopedBlockProperties(@ContentTypeID) as ScopedProperties ON 
				Property.ScopeName LIKE (ScopedProperties.ScopeName + '%')
				WHERE
                    ScopedProperties.ScopeName LIKE ('%.' + CAST(@PropertyDefinitionID as VARCHAR)+ '.')
                OR
                    ScopedProperties.ScopeName LIKE ('%.' + CAST(@PropertyDefinitionID as VARCHAR)+ '(')
	END
	
	RETURN 
END
GO

PRINT N'Altering Procedure [dbo].[netPropertyDefinitionCheckUsage]...';


GO
ALTER PROCEDURE [dbo].[netPropertyDefinitionCheckUsage]
(
	@PropertyDefinitionID	INT,
	@OnlyNoneMasterLanguage	BIT = 0,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	
	SET NOCOUNT ON
	
	--Get blocktype if property is block property
	DECLARE @ContentTypeID INT;
	SET @ContentTypeID = (SELECT tblContentType.pkID FROM 
		tblPropertyDefinition
		INNER JOIN tblPropertyDefinitionType ON tblPropertyDefinition.fkPropertyDefinitionTypeID = tblPropertyDefinitionType.pkID
		INNER JOIN tblContentType ON tblPropertyDefinitionType.fkContentTypeGUID = tblContentType.ContentTypeGUID
		WHERE tblPropertyDefinition.pkID = @PropertyDefinitionID);
	
	IF (@OnlyPublished = 1)
	BEGIN
		IF (@ContentTypeID IS NOT NULL)
		BEGIN
			SELECT TOP 1
				tblContentLanguage.fkContentID as ContentID, 
				0 AS WorkID
			FROM 
				tblContentProperty 
			INNER JOIN 
				dbo.GetExistingScopesForDefinition(@PropertyDefinitionID) as ExistingScopes ON tblContentProperty.ScopeName LIKE ExistingScopes.ScopeName + '%'
			INNER JOIN 
				tblContentLanguage ON tblContentProperty.fkContentID=tblContentLanguage.fkContentID
		END
		ELSE
		BEGIN
			SELECT TOP 1
				tblContentLanguage.fkContentID as ContentID, 
				0 AS WorkID
			FROM 
				tblContentProperty
			INNER JOIN 
				tblContentLanguage ON tblContentProperty.fkContentID=tblContentLanguage.fkContentID
			WHERE
				tblContentLanguage.fkLanguageBranchID=tblContentProperty.fkLanguageBranchID AND
				tblContentProperty.fkPropertyDefinitionID=@PropertyDefinitionID
		END
	END
	ELSE
	BEGIN
		IF (@ContentTypeID IS NOT NULL)
		BEGIN
			SELECT TOP 1
				tblWorkContent.fkContentID as ContentID, 
				tblWorkContent.pkID AS WorkID
			FROM 
				tblWorkContentProperty
			INNER JOIN
				dbo.GetExistingScopesForDefinition(@PropertyDefinitionID) as ExistingScopes ON tblWorkContentProperty.ScopeName LIKE ExistingScopes.ScopeName + '%'
			INNER JOIN
				tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
		END
		ELSE
		BEGIN
			SELECT TOP 1
				tblWorkContent.fkContentID as ContentID, 
				tblWorkContent.pkID AS WorkID
			FROM 
				tblWorkContentProperty
			INNER JOIN
				tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
			WHERE
				tblWorkContentProperty.fkPropertyDefinitionID=@PropertyDefinitionID
		END
	END

	RETURN 0
END
GO

PRINT N'Altering Procedure [dbo].[netPropertyDefinitionDelete]...';


GO
ALTER PROCEDURE [dbo].[netPropertyDefinitionDelete]
(
	@PropertyDefinitionID INT
)
AS
BEGIN
	DECLARE @ScopedProperties TABLE (ScopeName nvarchar(450))
	INSERT INTO @ScopedProperties SELECT * FROM dbo.GetExistingScopesForDefinition(@PropertyDefinitionID)

	DELETE Property FROM tblContentProperty Property
        INNER JOIN @ScopedProperties AS Scopes ON Property.ScopeName LIKE Scopes.ScopeName + '%'
    DELETE WorkProperty FROM tblWorkContentProperty WorkProperty
        INNER JOIN @ScopedProperties AS Scopes ON WorkProperty.ScopeName LIKE Scopes.ScopeName + '%'
    DELETE FROM tblInlineBlockUsage WHERE ScopeName IN (SELECT ScopeName FROM @ScopedProperties) 
	DELETE FROM tblPropertyDefault WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblProperty WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblWorkProperty WHERE fkPageDefinitionID=@PropertyDefinitionID
	DELETE FROM tblCategoryPage WHERE CategoryType=@PropertyDefinitionID
	DELETE FROM tblWorkCategory WHERE CategoryType=@PropertyDefinitionID
    DELETE FROM tblContentSoftlink WHERE fkOwnerPropertyDefinitionID=@PropertyDefinitionID
	DELETE FROM tblPageDefinition WHERE pkID=@PropertyDefinitionID
END
GO

PRINT N'Altering Procedure [dbo].[netPropertyDefinitionGetUsage]...';


GO
ALTER PROCEDURE [dbo].[netPropertyDefinitionGetUsage]
(
	@PropertyDefinitionID	INT,
	@OnlyNoneMasterLanguage	BIT = 0,
	@OnlyPublished	BIT = 0
)
AS
BEGIN
	
	
	SET NOCOUNT ON
	
	--Get blocktype if property is block property
	DECLARE @ContentTypeID INT;
	SET @ContentTypeID = (SELECT tblContentType.pkID FROM 
		tblPropertyDefinition
		INNER JOIN tblPropertyDefinitionType ON tblPropertyDefinition.fkPropertyDefinitionTypeID = tblPropertyDefinitionType.pkID
		INNER JOIN tblContentType ON tblPropertyDefinitionType.fkContentTypeGUID = tblContentType.ContentTypeGUID
		WHERE tblPropertyDefinition.pkID = @PropertyDefinitionID);
	
	IF (@OnlyPublished = 1)
	BEGIN
		IF (@ContentTypeID IS NOT NULL)
		BEGIN
			SELECT DISTINCT
				tblContentLanguage.fkContentID as ContentID, 
				0 AS WorkID,
				tblContentLanguage.Name,
				tblLanguageBranch.LanguageID AS LanguageBranch,
				tblLanguageBranch.Name AS LanguageBranchName
			FROM 
				tblContentProperty 
			INNER JOIN 
				dbo.GetExistingScopesForDefinition(@PropertyDefinitionID) as ExistingScopes ON tblContentProperty.ScopeName LIKE ExistingScopes.ScopeName + '%'
			INNER JOIN 
				tblContentLanguage ON tblContentProperty.fkContentID=tblContentLanguage.fkContentID
			INNER JOIN
				tblContent ON tblContent.pkID=tblContentLanguage.fkContentID
			INNER JOIN
				tblLanguageBranch ON tblLanguageBranch.pkID=tblContentLanguage.fkLanguageBranchID
			WHERE
				tblContentLanguage.fkLanguageBranchID=tblContentProperty.fkLanguageBranchID AND
				(@OnlyNoneMasterLanguage=0 OR tblContentProperty.fkLanguageBranchID<>tblContent.fkMasterLanguageBranchID)
		END
		ELSE
		BEGIN
			SELECT DISTINCT
				tblContentLanguage.fkContentID as ContentID, 
				0 AS WorkID,
				tblContentLanguage.Name,
				tblLanguageBranch.LanguageID AS LanguageBranch,
				tblLanguageBranch.Name AS LanguageBranchName
			FROM 
				tblContentProperty
			INNER JOIN 
				tblContentLanguage ON tblContentProperty.fkContentID=tblContentLanguage.fkContentID
			INNER JOIN
				tblContent ON tblContent.pkID=tblContentLanguage.fkContentID
			INNER JOIN
				tblLanguageBranch ON tblLanguageBranch.pkID=tblContentLanguage.fkLanguageBranchID
			WHERE
				tblContentLanguage.fkLanguageBranchID=tblContentProperty.fkLanguageBranchID AND
				tblContentProperty.fkPropertyDefinitionID=@PropertyDefinitionID AND
				(@OnlyNoneMasterLanguage=0 OR tblContentProperty.fkLanguageBranchID<>tblContent.fkMasterLanguageBranchID)
		END
	END
	ELSE
	BEGIN
		IF (@ContentTypeID IS NOT NULL)
		BEGIN
			SELECT DISTINCT
				tblContent.pkID as ContentID, 
				tblWorkContent.pkID AS WorkID,
				tblWorkContent.Name,
				tblLanguageBranch.LanguageID AS LanguageBranch
			FROM 
				tblWorkContentProperty
			INNER JOIN
				dbo.GetExistingScopesForDefinition(@PropertyDefinitionID) as ExistingScopes ON tblWorkContentProperty.ScopeName LIKE ExistingScopes.ScopeName + '%'
			INNER JOIN
				tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
			INNER JOIN
				tblContent ON tblWorkContent.fkContentID=tblContent.pkID
			INNER JOIN 
				tblContentLanguage ON tblWorkContent.fkContentID=tblContentLanguage.fkContentID 
			INNER JOIN
				tblLanguageBranch ON tblLanguageBranch.pkID=tblContentLanguage.fkLanguageBranchID
			WHERE
				tblWorkContent.fkLanguageBranchID = tblLanguageBranch.pkID AND
				(@OnlyNoneMasterLanguage=0 OR tblWorkContent.fkLanguageBranchID<>tblContent.fkMasterLanguageBranchID)
		END
		ELSE
		BEGIN
			SELECT DISTINCT
				tblContent.pkID as ContentID, 
				tblWorkContent.pkID AS WorkID,
				tblWorkContent.Name,
				tblLanguageBranch.LanguageID AS LanguageBranch
			FROM 
				tblWorkContentProperty
			INNER JOIN
				tblWorkContent ON tblWorkContentProperty.fkWorkContentID=tblWorkContent.pkID
			INNER JOIN
				tblContent ON tblWorkContent.fkContentID=tblContent.pkID
			INNER JOIN 
				tblContentLanguage ON tblWorkContent.fkContentID=tblContentLanguage.fkContentID 
			INNER JOIN
				tblLanguageBranch ON tblLanguageBranch.pkID=tblContentLanguage.fkLanguageBranchID
			WHERE
				tblWorkContent.fkLanguageBranchID = tblLanguageBranch.pkID AND
				tblWorkContentProperty.fkPropertyDefinitionID=@PropertyDefinitionID AND
				(@OnlyNoneMasterLanguage=0 OR tblWorkContent.fkLanguageBranchID<>tblContent.fkMasterLanguageBranchID)
		END
	END

	RETURN 0
END
GO

ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8014
GO

PRINT N'Update complete.';
GO
