--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8003)
				select 0, 'Already correct database version'
            else if (@ver = 8002)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery

GO
PRINT N'Creating Table [dbo].[tblSynchedApplication]...';


GO
CREATE TABLE [dbo].[tblSynchedApplication] (
    [pkID]                   INT            IDENTITY (1, 1) NOT NULL,
    [ApplicationName]        NVARCHAR (255) NOT NULL,
    [LoweredApplicationName] NVARCHAR (255) NOT NULL,
    CONSTRAINT [PK_tblSynchedApplication] PRIMARY KEY CLUSTERED ([pkID] ASC)
);


GO
PRINT N'Creating Index [dbo].[tblSynchedApplication].[IX_tblSynchedApplication_Unique]...';


GO
CREATE UNIQUE NONCLUSTERED INDEX [IX_tblSynchedApplication_Unique]
    ON [dbo].[tblSynchedApplication]([LoweredApplicationName] ASC);


GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';


GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8003
GO
PRINT N'Creating Procedure [dbo].[netSynchedApplicationInsertOrUpdate]...';


GO

CREATE PROCEDURE dbo.netSynchedApplicationInsertOrUpdate 
(
	@ApplicationName NVARCHAR(255)
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	DECLARE @ApplicationID INT
	DECLARE @LoweredName NVARCHAR(255)
	SET @LoweredName=LOWER(@ApplicationName)

    -- Updating is unnecessary in this case since we only have the name. But this proc is prepared for future
    -- updates where an application might have more data.
	SET @ApplicationID = (SELECT pkID FROM [tblSynchedApplication] WHERE LoweredApplicationName=@LoweredName)
	IF (@ApplicationID IS NOT NULL)
	BEGIN
		UPDATE [tblSynchedApplication] SET
			ApplicationName = @ApplicationName,
			LoweredApplicationName = @LoweredName
		WHERE 
			pkID = @ApplicationID
	END
	ELSE
	BEGIN
		INSERT INTO [tblSynchedApplication] 
			(ApplicationName, LoweredApplicationName) 
		SELECT 
			@ApplicationName, 
			@LoweredName

		SET @ApplicationID= SCOPE_IDENTITY()
	END

	SELECT @ApplicationID
END
GO
PRINT N'Creating Procedure [dbo].[netSynchedApplicationList]...';


GO
CREATE PROCEDURE dbo.netSynchedApplicationList
(
	@ApplicationNameToMatch NVARCHAR(255) = NULL,
	@StartIndex	INT,
	@MaxCount	INT
)
AS
BEGIN
	SET @ApplicationNameToMatch = LOWER(@ApplicationNameToMatch);

	WITH MatchedSynchedApplicationsCTE
	AS
	(
		SELECT 
		ROW_NUMBER() OVER (ORDER BY ApplicationName) AS RowNum, ApplicationName
		FROM
		(	
			SELECT
				pkID, ApplicationName
			FROM
				[tblSynchedApplication]
			WHERE
				(@ApplicationNameToMatch IS NULL) OR 
				([tblSynchedApplication].LoweredApplicationName LIKE @ApplicationNameToMatch + '%')
		)
		AS Result
	)

	SELECT TOP(@MaxCount) ApplicationName, (SELECT COUNT(*) FROM MatchedSynchedApplicationsCTE) AS 'TotalCount'
		FROM MatchedSynchedApplicationsCTE 
		WHERE RowNum BETWEEN (@StartIndex - 1) * @MaxCount + 1 AND @StartIndex * @MaxCount 
		ORDER BY ApplicationName
END
GO
PRINT N'Update complete.';


GO
