--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8006)
				select 0, 'Already correct database version'
            else if (@ver = 8005)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery
GO

PRINT N'Altering [dbo].[tblSynchedUser]...';
GO
ALTER TABLE [dbo].[tblSynchedUser]
    ADD [LastSynced] DATETIME NULL;


PRINT N'Altering [dbo].[netSynchedUserInsertOrUpdate]...';
GO
ALTER PROCEDURE dbo.netSynchedUserInsertOrUpdate 
(
	@UserName NVARCHAR(255),
	@GivenName NVARCHAR(255) = NULL,
	@Surname NVARCHAR(255) = NULL,
	@Email NVARCHAR(255) = NULL,
	@Metadata NVARCHAR(MAX) = NULL,
    @Saved DATETIME
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	DECLARE @UserID INT
	DECLARE @LoweredName NVARCHAR(255)
	SET @LoweredName=LOWER(@UserName)

	SET @UserID = (SELECT pkID FROM [tblSynchedUser] WHERE LoweredUserName=@LoweredName)
	IF (@UserID IS NOT NULL)
	BEGIN
		UPDATE [tblSynchedUser] SET
			UserName = @UserName,
			LoweredUserName = @LoweredName,
			Email =  LOWER(@Email),
			GivenName = @GivenName,
			LoweredGivenName = LOWER(@GivenName),
			Surname = @Surname,
			LoweredSurname = LOWER(@Surname),
			Metadata = @Metadata,
            LastSynced = @Saved
		WHERE 
			pkID = @UserID
	END
	ELSE
	BEGIN
		INSERT INTO [tblSynchedUser] 
			(UserName, LoweredUserName, Email, GivenName, LoweredGivenName, Surname, LoweredSurname, Metadata, LastSynced) 
		SELECT 
			@UserName, 
			@LoweredName,
			Lower(@Email),
			@GivenName,
			Lower(@GivenName),
			@Surname,
			Lower(@Surname),
			@Metadata,
            @Saved

		SET @UserID= SCOPE_IDENTITY()
	END

	SELECT @UserID
END


GO
PRINT N'Altering [dbo].[netSynchedUserRoleUpdate]...';
GO
ALTER PROCEDURE dbo.netSynchedUserRoleUpdate
(
	@UserName NVARCHAR(255),
	@Roles dbo.StringParameterTable READONLY,
    @RolesHash INT = NULL,
    @Saved DATETIME
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	DECLARE @UserID INT
    DECLARE @CurrentRolesHash INT
	SELECT @UserID = pkID, @CurrentRolesHash = RolesHash FROM [tblSynchedUser] WHERE LoweredUserName = LOWER(@UserName)
	IF (@UserID IS NULL)
	BEGIN
		RAISERROR(N'No user with username %s was found', 16, 1, @UserName)
	END
    
    IF ((@RolesHash IS NULL AND @CurrentRolesHash IS NOT NULL) OR (@RolesHash IS NOT NULL AND @CurrentRolesHash IS NULL) OR @RolesHash <>  @CurrentRolesHash)
    BEGIN
        UPDATE [tblSynchedUser] SET RolesHash = @RolesHash WHERE pkID = @UserID

	    /*First ensure roles are in role table*/
	    MERGE [tblSynchedUserRole] AS Target
		    USING @Roles AS Source
		    ON (Target.LoweredRoleName = LOWER(Source.String))
		    WHEN NOT MATCHED BY Target THEN
			    INSERT (RoleName, LoweredRoleName)
			    VALUES (Source.String, LOWER(Source.String));

	    /* Remove all existing fole for user */
	    DELETE FROM [tblSynchedUserRelations] WHERE [fkSynchedUser] = @UserID

	    /* Insert roles */
	    INSERT INTO [tblSynchedUserRelations] ([fkSynchedRole], [fkSynchedUser])
	    SELECT [tblSynchedUserRole].pkID, @UserID FROM 
	    [tblSynchedUserRole] INNER JOIN @Roles AS R ON [tblSynchedUserRole].LoweredRoleName = LOWER(R.String)
    END

    /* set last synced every time user has logged in */
    UPDATE [tblSynchedUser] SET LastSynced = @Saved WHERE pkID = @UserID
END



GO
PRINT N'Altering Procedure [dbo].[netSynchedUserGetMetadata]...';
GO
ALTER PROCEDURE dbo.netSynchedUserGetMetadata
(
	@UserName NVARCHAR(255)
)
AS
BEGIN
	SET @UserName = LOWER(@UserName)
	SELECT Email, GivenName, Surname,LastSynced, Metadata FROM [tblSynchedUser]
	WHERE LoweredUserName = @UserName
END


GO
PRINT N'Creating Procedure [dbo].[netSynchedUserDeleteByUserName]...';
GO
CREATE PROCEDURE dbo.netSynchedUserDeleteByUserName
(
	@UserName NVARCHAR(255)
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON
	
	DECLARE @UserID INT
	SET @UserID = NULL

	SELECT
		@UserID = pkID
	FROM
		[tblSynchedUser]
	WHERE
		[UserName] = @UserName
	
	/* user does not exist - do nothing */	
    IF (@UserID IS NULL)
    BEGIN
        RETURN
    END

    DELETE FROM
        [tblSynchedUserRelations]
    WHERE
        fkSynchedUser = @UserID

    DELETE FROM
        [tblSynchedUser]
    WHERE
        [pkID] = @UserID
 
END


GO
PRINT N'Creating Procedure [dbo].[netSynchedUserDeleteInactiveUsers]...';
GO
CREATE PROCEDURE [dbo].[netSynchedUserDeleteInactiveUsers](
	@LastSyncedBefore DATETIME)
AS
BEGIN
	DECLARE @RemoveUsers TABLE (ID INT, UserName NVARCHAR(255))

	INSERT INTO @RemoveUsers
	SELECT usr.pkID, usr.UserName
	FROM [dbo].[tblSynchedUser] usr
	WHERE usr.LastSynced < @LastSyncedBefore

	IF EXISTS(SELECT 1 FROM @RemoveUsers)  
	BEGIN
        DELETE relate FROM [dbo].[tblSynchedUserRelations] relate
		JOIN @RemoveUsers rm ON relate.fkSynchedUser = rm.ID

		DELETE usr FROM [dbo].[tblSynchedUser] usr
		JOIN @RemoveUsers rm ON usr.pkID = rm.ID
	END

	SELECT UserName from @RemoveUsers;
END



GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';
GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8006
GO


PRINT N'Update complete.';

GO
