--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8008)
				select 0, 'Already correct database version'
            else if (@ver = 8007)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery
GO


PRINT N'Altering Procedure [dbo].[netQuickSearchByExternalUrl]...';
GO

ALTER PROCEDURE dbo.netQuickSearchByExternalUrl
(
    @Url	NVARCHAR(255),
    @CurrentTime	DATETIME
)
    AS
BEGIN
	SET NOCOUNT ON

	DECLARE @LoweredUrl NVARCHAR(255)

	SET @LoweredUrl = Lower(@Url)

	/*
		Performance notes: The subquery "Pages" must not have any more predicates or return the values used in the outer WHERE-clause, otherwise
		SQL Server falls back to a costly index scan. The performance hints LOOP on the joins are also required for the same reason, the resultset
		from "Pages" is so small that a loop join is superior in performance to index scan/hash match, a factor 1000x.
	*/

SELECT
    tblPageLanguage.fkPageID,
    tblLanguageBranch.LanguageID as LanguageBranch,
    tblPageLanguage.ExternalURL
FROM
    (
        SELECT fkPageID,fkLanguageBranchID
        FROM tblPageLanguage
        WHERE (tblPageLanguage.ExternalURL = @LoweredUrl) OR (tblPageLanguage.ExternalURL LIKE @LoweredUrl + '/%')
    ) AS Pages
    INNER LOOP JOIN
		tblPage ON tblPage.pkID = Pages.fkPageID
    INNER LOOP JOIN
    tblPageLanguage ON tblPageLanguage.fkPageID=Pages.fkPageID AND tblPageLanguage.fkLanguageBranchID=Pages.fkLanguageBranchID
    INNER LOOP JOIN
    tblLanguageBranch ON tblLanguageBranch.pkID = Pages.fkLanguageBranchID
WHERE
    tblPage.Deleted=0 AND
    tblPageLanguage.[Status]=4 AND
    tblPageLanguage.StartPublish <= @CurrentTime AND
    (tblPageLanguage.StopPublish IS NULL OR tblPageLanguage.StopPublish >= @CurrentTime)
ORDER BY
    tblPageLanguage.Changed DESC
END
GO


PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';
GO

ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8008
GO


PRINT N'Update complete.';
GO
