--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 8009)
				select 0, 'Already correct database version'
            else if (@ver = 8008)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery
GO

PRINT N'Creating User-Defined Table Type [dbo].[SavePropertyTableType]...';


GO
CREATE TYPE [dbo].[SavePropertyTableType] AS TABLE (
    [PropertyDefinitionID] INT              NULL,
    [ScopeName]            NVARCHAR (450)   NULL,
    [Number]               INT              NULL,
    [Boolean]              BIT              NULL,
    [Date]                 DATETIME         NULL,
    [FloatNumber]          FLOAT (53)       NULL,
    [ContentType]          INT              NULL,
    [String]               NVARCHAR (450)   NULL,
    [LinkGuid]             UNIQUEIDENTIFIER NULL,
    [ContentLink]          INT              NULL,
    [LongString]           NVARCHAR (MAX)   NULL);


GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';


GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 8009
GO
PRINT N'Creating Procedure [dbo].[netPropertySaveBatch]...';


GO

CREATE PROCEDURE [dbo].[netPropertySaveBatch]
(
	@ContentID		    INT,
	@WorkContentID		INT,
	@LanguageBranch		NCHAR(17) = NULL,
	@PropertyValues     dbo.SavePropertyTableType READONLY
)
AS
BEGIN
	SET NOCOUNT ON
	SET XACT_ABORT ON

	DECLARE @LangBranchID NCHAR(17);
	IF (@WorkContentID <> 0)
		SELECT @LangBranchID = fkLanguageBranchID FROM tblWorkContent WHERE pkID = @WorkContentID
	ELSE
		SELECT @LangBranchID = pkID FROM tblLanguageBranch WHERE LanguageID = @LanguageBranch

	IF @LangBranchID IS NULL 
	BEGIN 
		if @LanguageBranch IS NOT NULL
			RAISERROR('Language branch %s is not defined',16,1, @LanguageBranch)
		else
			SET @LangBranchID = 1
	END

	DECLARE @IsLanguagePublished BIT;
	IF EXISTS(SELECT fkContentID FROM tblContentLanguage 
		WHERE fkContentID = @ContentID AND fkLanguageBranchID = CAST(@LangBranchID AS INT) AND Status = 4)
		SET @IsLanguagePublished = 1
	ELSE
		SET @IsLanguagePublished = 0
	

	IF (@WorkContentID IS NOT NULL)
	BEGIN				
		/* Insert or update property */
	    MERGE tblWorkContentProperty AS Target
        USING @PropertyValues AS SOURCE
        ON (Target.fkWorkContentID=@WorkContentID AND Target.fkPropertyDefinitionID = SOURCE.PropertyDefinitionID
        AND ((SOURCE.ScopeName IS NULL AND Target.ScopeName IS NULL) OR (SOURCE.ScopeName = Target.ScopeName)))
		WHEN NOT MATCHED BY Target THEN
            INSERT (fkWorkContentID,
				fkPropertyDefinitionID,
				ScopeName,
				Number,
				Boolean,
				[Date],
				FloatNumber,
				ContentType,
				String,
				LinkGuid,
				ContentLink,
				LongString)
            VALUES (@WorkContentID,
				SOURCE.PropertyDefinitionID,
				SOURCE.ScopeName,
				SOURCE.Number,
				SOURCE.Boolean,
				SOURCE.[Date],
				SOURCE.FloatNumber,
				SOURCE.ContentType,
				SOURCE.String,
				SOURCE.LinkGuid,
				SOURCE.ContentLink,
				SOURCE.LongString)
        WHEN MATCHED THEN
            UPDATE SET
                ScopeName = SOURCE.ScopeName,
				Number = SOURCE.Number,
				Boolean = SOURCE.Boolean,
				[Date] = SOURCE.[Date],
				FloatNumber = SOURCE.FloatNumber,
				ContentType = SOURCE.ContentType,
				String = SOURCE.String,
				LinkGuid = SOURCE.LinkGuid,
				ContentLink = SOURCE.ContentLink,
				LongString = SOURCE.LongString;
	END
		
	/* For published or languages where no version is published we save value in tblContentProperty as well. Reason for this is that if when page is loaded
	through tblContentProperty (typically netPageListPaged) the page gets populated correctly. */
	IF (@WorkContentID IS NULL OR @IsLanguagePublished = 0)
	BEGIN
		/* Insert or update property */
		MERGE tblContentProperty AS Target
        USING @PropertyValues AS SOURCE
        ON (Target.fkContentID = @ContentID AND Target.fkPropertyDefinitionID = SOURCE.PropertyDefinitionID  AND
				((SOURCE.ScopeName IS NULL AND Target.ScopeName IS NULL) OR (SOURCE.ScopeName = Target.ScopeName)) AND @LangBranchID = Target.fkLanguageBranchID)
        WHEN MATCHED THEN
            UPDATE SET
                ScopeName = SOURCE.ScopeName,
				Number = SOURCE.Number,
				Boolean = SOURCE.Boolean,
				[Date] = SOURCE.[Date],
				FloatNumber = SOURCE.FloatNumber,
				ContentType = SOURCE.ContentType,
				String = SOURCE.String,
				LinkGuid = SOURCE.LinkGuid,
				ContentLink = SOURCE.ContentLink,
				LongString = SOURCE.LongString,
                /* LongString is utf-16 - Datalength gives bytes, i e div by 2 gives characters */
		        /* Include length to handle delayed loading of LongString with threshold */
		        LongStringLength = COALESCE(DATALENGTH(SOURCE.LongString), 0) / 2
        WHEN NOT MATCHED THEN
            INSERT (fkContentID,
				fkPropertyDefinitionID,
				ScopeName,
				Number,
				Boolean,
				[Date],
				FloatNumber,
				ContentType,
				String,
				LinkGuid,
				ContentLink,
				LongString,
                LongStringLength,
				fkLanguageBranchID)
            VALUES (@ContentID,
				SOURCE.PropertyDefinitionID,
				SOURCE.ScopeName,
				SOURCE.Number,
				SOURCE.Boolean,
				SOURCE.Date,
				SOURCE.FloatNumber,
				SOURCE.ContentType,
                SOURCE.String,
				SOURCE.LinkGuid,
				SOURCE.ContentLink,
				SOURCE.LongString,
                COALESCE(DATALENGTH(SOURCE.LongString), 0) / 2,
				@LangBranchID);
			
				
	END
END
GO
PRINT N'Update complete.';


GO
